/*
 * ============================================================================
 * GNU Lesser General Public License
 * ============================================================================
 *
 * Square Brackets - Java Array Framework.
 * Copyright (C) 2013  Beatgrid Media B.V.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 *
 * Beatgrid Media B.V.
 * Sisalbaan 5-A
 * 2352 AZ Leiderdorp
 * The Netherlands
 *
 * info@squarebrackets.org
 * http://squarebrackets.org
 */
package org.squarebrackets;

import org.squarebrackets.function.BooleanConsumer;
import org.squarebrackets.function.BooleanPredicate;
import org.squarebrackets.function.BooleanUnaryOperator;
import org.squarebrackets.function.ByteConsumer;
import org.squarebrackets.function.BytePredicate;
import org.squarebrackets.function.ByteUnaryOperator;
import org.squarebrackets.function.CharConsumer;
import org.squarebrackets.function.CharPredicate;
import org.squarebrackets.function.CharUnaryOperator;
import org.squarebrackets.function.FloatConsumer;
import org.squarebrackets.function.FloatPredicate;
import org.squarebrackets.function.FloatUnaryOperator;
import org.squarebrackets.function.ShortConsumer;
import org.squarebrackets.function.ShortPredicate;
import org.squarebrackets.function.ShortUnaryOperator;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.io.IOException;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.Collection;
import java.util.Comparator;
import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.List;
import java.util.NavigableSet;
import java.util.NoSuchElementException;
import java.util.Objects;
import java.util.RandomAccess;
import java.util.Spliterator;
import java.util.Spliterators;
import java.util.function.Consumer;
import java.util.function.DoubleConsumer;
import java.util.function.DoublePredicate;
import java.util.function.DoubleUnaryOperator;
import java.util.function.IntConsumer;
import java.util.function.IntFunction;
import java.util.function.IntPredicate;
import java.util.function.IntUnaryOperator;
import java.util.function.LongConsumer;
import java.util.function.LongPredicate;
import java.util.function.LongUnaryOperator;
import java.util.function.Predicate;
import java.util.function.Supplier;
import java.util.function.UnaryOperator;
import java.util.stream.DoubleStream;
import java.util.stream.IntStream;
import java.util.stream.LongStream;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

import static java.lang.Math.max;
import static java.lang.Math.min;
import static java.lang.System.arraycopy;
import static org.squarebrackets.ArrayCharacteristics.DISTINCT;
import static org.squarebrackets.ArrayCharacteristics.MUTABLE;
import static org.squarebrackets.ArrayCharacteristics.NONNULL;
import static org.squarebrackets.ArrayCharacteristics.SORTED;

/**
 * This class contains various methods for manipulating arrays (such as sorting and searching).
 *
 * @author Leon van Zantvoort
 */
public final class Arrays {

    /**
     * Suppresses default constructor, ensuring non-instantiability.
     */
    private Arrays() {}

    /**
     * <p>Returns an immutable array backed by the specified {@code list}. All operations on the resulting array are
     * delegated to the list.</p>
     *
     * <p>The list does not need to be empty at the time this method is invoked. The list must not be modified directly
     * after this method returns.</p>
     *
     * @param list the list by which the array will be backed.
     * @param <E> element type.
     * @return an immutable array backed by the specified list.
     * @throws NullPointerException if the specified list is null.
     */
    public static <E> ObjectArray<E> newArray(@Nonnull List<E> list) {
        return ArrayBridge.newArray(list);
    }

    /**
     * <p>Returns a mutable array backed by the specified {@code list}. All operations on the resulting array are
     * delegated to the list.</p>
     *
     * <p>The list does not need to be empty at the time this method is invoked. The list must not be modified directly
     * after this method returns.</p>
     *
     * @param list the list by which the array will be backed.
     * @param <E> element type.
     * @return a mutable array backed by the specified list.
     * @throws NullPointerException if the specified list is null.
     */
    public static <E> MutableObjectArray<E> newMutableArray(@Nonnull List<E> list) {
        return ArrayBridge.newMutableArray(list);
    }

    /**
     * <p>Returns a dynamic array backed by the specified {@code list}. All operations on the resulting array are
     * delegated to the list.</p>
     *
     * <p>The list does not need to be empty at the time this method is invoked. The list must not be modified directly
     * after this method returns.</p>
     *
     * @param list the list by which the array will be backed.
     * @param <E> element type.
     * @return a dynamic array backed by the specified list.
     * @throws NullPointerException if the specified list is null.
     */
    public static <E> DynamicObjectArray<E> newDynamicArray(@Nonnull List<E> list) {
        return ArrayBridge.newDynamicArray(list);
    }

    /**
     * Returns an immutable copy of the elements of the specified {@code collection}. The copy is obtained through the
     * {@code collection}'s {@code toArray} method.
     *
     * @param collection collection to be copied.
     * @param <E> element type.
     * @return an immutable copy of the specified collection.
     * @throws NullPointerException if the specified collection is null.
     */
    public static <E> Array<E> copyOf(@Nonnull Collection <E> collection) {
        return ArrayBridge.copyOf(collection);
    }

    /**
     * Returns a list backed by the specified array. This method acts as bridge between array-based and collection-based
     * APIs. The returned list is serializable and implements {@link RandomAccess}.
     *
     * <p>If the specified {@code array} is immutable, the returned list is an unmodifiable view of the specified
     * {@code} array. Query operations on the returned list "read through" to the specified {@code array}. Element
     * modification, addition or removal results in an {@code UnsupportedOperationException}.</p>
     *
     * <p>If the specified array is mutable, the returned list is fixed-length. Query operations on the returned list
     * "read through" to the specified {@code array}. Modifying operations on the returned list "write through" to the
     * specified {@code array}. Element addition or removal results in an {@code UnsupportedOperationException}.</p>
     *
     * <p>If the specified array is dynamic, the returned list is a fully functional view of the specified {@code array}.
     * "write through" to the specified {@code array}. Query operations on the returned list "read through" to the
     * specified {@code array}. Modifying, addition and removal operations on the returned list "write through" to the
     * specified {@code array}.
     *
     * @param <E> element type.
     * @param array the array by which the list will be backed.
     * @return a list view of the specified array.
     * @throws NullPointerException if the specified array is null.
     */
    public static <E> List<E> asList(@Nonnull Array<E> array) {
        return ArrayBridge.asList(array);
    }

    /**
     * Returns a navigable set backed by the specified array. The array can be modified by the navigable set
     * implementation to ensure that it adheres to the semantics of the navigable set. This method acts as bridge
     * between array-based and collection-based APIs. The returned navigable set is serializable.
     *
     * @param <E> element type.
     * @param array the array by which the list will be backed.
     * @return a list view of the specified array.
     * @throws NullPointerException if the specified array is null.
     */
    public static <E> NavigableSet<E> asNavigableSet(@Nonnull DynamicArray<E> array) {
        return ArrayBridge.asNavigableSet(array);
    }

    /**
     * Returns a navigable set backed by the specified array. The array can be modified by the navigable set
     * implementation to ensure that it adheres to the semantics of the navigable set. This method acts as bridge
     * between array-based and collection-based APIs. The returned navigable set is serializable.
     *
     * @param array the list by which the array will be backed.
     * @param comparator comparator used for sorting set.
     * @param <E> element type.
     * @return an immutable copy of the specified collection.
     * @throws NullPointerException if the specified collection is null.
     */
    public static <E> NavigableSet<E> asNavigableSet(@Nonnull DynamicObjectArray<E> array, @Nullable Comparator<? super E> comparator) {
        return ArrayBridge.asNavigableSet(array, comparator);
    }

    /**
     * Comparator for natural ordering of objects. Comparator throws a {@code ClassCastException} if objects are not
     * (mutually) {@code Comparable}.
     */
    @SuppressWarnings("unchecked")
    static final Comparator<Object> NATURAL_ORDER_COMPARATOR = (left, right) -> ((Comparable<Object>) left).compareTo(right);

    /**
     * Checks if the given index is in range.  If not, throws an appropriate
     * runtime exception.  This method does *not* check if the index is
     * negative: It is always used immediately prior to an array access,
     * which throws an IndexOutOfBoundsException if index is negative.
     */
    static void rangeCheck(int arrayLength, int index) {
        if (index >= arrayLength || index < 0) {
            throw new IndexOutOfBoundsException(outOfBoundsMsg(arrayLength, index));
        }
    }

    /**
     * Checks that {@code fromIndex} and {@code index} are in
     * the range and throws an appropriate exception, if they aren't.
     */
    private static void rangeCheckFirst(int length, int index) {
        if (index < 0) {
            throw new ArrayIndexOutOfBoundsException(index);
        }
        if (index > length) {
            throw new ArrayIndexOutOfBoundsException(index);
        }
    }

    /**
     * Checks that {@code index} and {@code toIndex} are in
     * the range and throws an appropriate exception, if they aren't.
     */
    private static void rangeCheckLast(int length, int index) {
        if (index >= length) {
            throw new ArrayIndexOutOfBoundsException(index);
        }
        if (index < 0) {
            throw new ArrayIndexOutOfBoundsException(index);
        }
    }

    /**
     * A version of rangeCheck used by add and addAll.
     *
     * @throws IndexOutOfBoundsException if {@code index} position is invalid.
     */
    static void rangeCheckForAdd(int arrayLength, int index) {
        if (index > arrayLength || index < 0) {
            throw new IndexOutOfBoundsException(outOfBoundsMsg(arrayLength, index));
        }
    }

    /**
     * Constructs an IndexOutOfBoundsException detail message.
     * Of the many possible refactorings of the error handling code,
     * this "outlining" performs best with both server and client VMs.
     */
    static String outOfBoundsMsg(int arrayLength, int index) {
        return String.format("Index: %d, Length: %d", index, arrayLength);
    }

    /**
     * Checks if the given boundary is within this array's boundaries.
     */
    static void subArrayCheck(int arrayOffset, int arrayLength, int offset, int length) {
        if (offset < arrayOffset || offset + length > arrayOffset + arrayLength) {
            throw new IndexOutOfBoundsException(outOfBoundsMsg(arrayOffset, arrayLength, offset, length));
        }
    }

    /**
     * Constructs an IndexOutOfBoundsException detail message.
     * Of the many possible refactorings of the error handling code,
     * this "outlining" performs best with both server and client VMs.
     */
    static String outOfBoundsMsg(int arrayOffset, int arrayLength, int offset, int length) {
        return String.format("Offset: %d, Length: %d, SubArray Offset: %d, SubArray Length: %d", arrayOffset, arrayLength, offset, length);
    }

    /**
     * The maximum size of array to allocate. Some VMs reserve some header words in an array. Attempts to allocate
     * larger arrays may result in OutOfMemoryError: Requested array size exceeds VM limit.
     */
    static final int MAX_ARRAY_SIZE = Integer.MAX_VALUE - 8;

    /**
     * Returns huge array size.
     *
     * @param minCapacity min size to return.
     * @return max array size.
     */
    static int hugeCapacity(int minCapacity) {
        if (minCapacity < 0) // Overflow.
            throw new OutOfMemoryError();
        return (minCapacity > MAX_ARRAY_SIZE) ?
                Integer.MAX_VALUE :
                MAX_ARRAY_SIZE;
    }

    /**
     * Returns {@code true} if the two specified arrays are equal to one another. Two arrays are considered equal if
     * both arrays contain the same number of elements, and all corresponding pairs of elements in the two arrays are
     * equal. In other words, two arrays are equal if they contain the same elements in the same order. Primitive
     * values are regarded equal to their boxed object counterparts. Also, two array references are considered equal if
     * both are {@code null}.
     *
     * @param a1 one array to be tested for equality.
     * @param a2 the other array to be tested for equality.
     * @return {@code true} if the two arrays are equal, {@code false} otherwise.
     */
    static boolean equals(Array<?> a1, Array<?> a2) {
        if (a1 == a2) {
            return true;
        }
        if (a1 == null || a2 == null) {
            return false;
        }
        if (a1.length() == a2.length()) {
            Iterator i1 = a1.iterator();
            Iterator i2 = a2.iterator();
            while (i1.hasNext()) {
                if (!Objects.equals(i1.next(), i2.next())) {
                    return false;
                }
            }
            return true;
        }
        return false;
    }

    /**
     * Creates a new array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static BooleanArray newArray(@Nonnull boolean[] array, int offset, int length, int characteristics, boolean copy) {
        return new BooleanArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new mutable array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static MutableBooleanArray newMutableArray(@Nonnull boolean[] array, int offset, int length, int characteristics, boolean copy) {
        return new MutableBooleanArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new dynamic array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static DynamicBooleanArray newDynamicArray(@Nonnull boolean[] array, int offset, int length, int characteristics, boolean copy) {
        return new DynamicBooleanArrayImpl(array, offset, length, characteristics, copy);
    }
    
    /**
     * Creates a new array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static ByteArray newArray(@Nonnull byte[] array, int offset, int length, int characteristics, boolean copy) {
        return new ByteArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new mutable array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static MutableByteArray newMutableArray(@Nonnull byte[] array, int offset, int length, int characteristics, boolean copy) {
        return new MutableByteArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new dynamic array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static DynamicByteArray newDynamicArray(@Nonnull byte[] array, int offset, int length, int characteristics, boolean copy) {
        return new DynamicByteArrayImpl(array, offset, length, characteristics, copy);
    }
    
    /**
     * Creates a new array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static CharArray newArray(@Nonnull char[] array, int offset, int length, int characteristics, boolean copy) {
        return new CharArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new mutable array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static MutableCharArray newMutableArray(@Nonnull char[] array, int offset, int length, int characteristics, boolean copy) {
        return new MutableCharArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new dynamic array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static DynamicCharArray newDynamicArray(@Nonnull char[] array, int offset, int length, int characteristics, boolean copy) {
        return new DynamicCharArrayImpl(array, offset, length, characteristics, copy);
    }
    
    /**
     * Creates a new array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static ShortArray newArray(@Nonnull short[] array, int offset, int length, int characteristics, boolean copy) {
        return new ShortArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new mutable array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static MutableShortArray newMutableArray(@Nonnull short[] array, int offset, int length, int characteristics, boolean copy) {
        return new MutableShortArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new dynamic array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static DynamicShortArray newDynamicArray(@Nonnull short[] array, int offset, int length, int characteristics, boolean copy) {
        return new DynamicShortArrayImpl(array, offset, length, characteristics, copy);
    }
    
    /**
     * Creates a new array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static IntArray newArray(@Nonnull int[] array, int offset, int length, int characteristics, boolean copy) {
        return new IntArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new mutable array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static MutableIntArray newMutableArray(@Nonnull int[] array, int offset, int length, int characteristics, boolean copy) {
        return new MutableIntArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new dynamic array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static DynamicIntArray newDynamicArray(@Nonnull int[] array, int offset, int length, int characteristics, boolean copy) {
        return new DynamicIntArrayImpl(array, offset, length, characteristics, copy);
    }
    
    /**
     * Creates a new array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static LongArray newArray(@Nonnull long[] array, int offset, int length, int characteristics, boolean copy) {
        return new LongArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new mutable array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static MutableLongArray newMutableArray(@Nonnull long[] array, int offset, int length, int characteristics, boolean copy) {
        return new MutableLongArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new dynamic array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static DynamicLongArray newDynamicArray(@Nonnull long[] array, int offset, int length, int characteristics, boolean copy) {
        return new DynamicLongArrayImpl(array, offset, length, characteristics, copy);
    }
    
    /**
     * Creates a new array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static FloatArray newArray(@Nonnull float[] array, int offset, int length, int characteristics, boolean copy) {
        return new FloatArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new mutable array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static MutableFloatArray newMutableArray(@Nonnull float[] array, int offset, int length, int characteristics, boolean copy) {
        return new MutableFloatArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new dynamic array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static DynamicFloatArray newDynamicArray(@Nonnull float[] array, int offset, int length, int characteristics, boolean copy) {
        return new DynamicFloatArrayImpl(array, offset, length, characteristics, copy);
    }    
    
    /**
     * Creates a new array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static DoubleArray newArray(@Nonnull double[] array, int offset, int length, int characteristics, boolean copy) {
        return new DoubleArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new mutable array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static MutableDoubleArray newMutableArray(@Nonnull double[] array, int offset, int length, int characteristics, boolean copy) {
        return new MutableDoubleArrayImpl(array, offset, length, characteristics, copy);
    }

    /**
     * Creates a new dynamic array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static DynamicDoubleArray newDynamicArray(@Nonnull double[] array, int offset, int length, int characteristics, boolean copy) {
        return new DynamicDoubleArrayImpl(array, offset, length, characteristics, copy);
    }
    
    /**
     * Creates a new array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static <E> ObjectArray<E> newArray(@Nonnull E[] array, int offset, int length, int characteristics, @Nullable Comparator<? super E> comparator, boolean copy) {
        return new ObjectArrayImpl<>(array, offset, length, characteristics, comparator, copy);
    }

    /**
     * Creates a new mutable array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static <E> MutableObjectArray<E> newMutableArray(@Nonnull E[] array, int offset, int length, int characteristics, @Nullable Comparator<? super E> comparator, boolean copy) {
        return new MutableObjectArrayImpl<>(array, offset, length, characteristics, comparator, copy);
    }

    /**
     * Creates a new dynamic array instance for specified parameters.
     *
     * @param array backing array.
     * @param offset start index of range.
     * @param length length of range.
     * @param characteristics array characteristics.
     * @param copy {@code true} to make a defensive copy of {@code array}, {@code false} to use {@code array} reference
     *             as backing array.
     * @return new array instance.
     */
    static <E> DynamicObjectArray<E> newDynamicArray(@Nonnull E[] array, int offset, int length, int characteristics, @Nullable Comparator<? super E> comparator, boolean copy) {
        return new DynamicObjectArrayImpl<>(array, offset, length, characteristics, comparator, copy);
    }    

    /**
     * Returns spliterator characteristics of array.
     *
     * @param array array to obtain characteristics from.
     * @param <E> element type.
     * @return spliterator characteristics.
     */
    static <E> int getCharacteristics(@Nonnull Array<E> array) {
        return Spliterator.ORDERED
                | Spliterator.SIZED
                | Spliterator.SUBSIZED
                | (!array.hasCharacteristics(MUTABLE) ? Spliterator.IMMUTABLE : 0)
                | (array.hasCharacteristics(NONNULL) ? Spliterator.NONNULL : 0)
                | (array.hasCharacteristics(SORTED) ? Spliterator.SORTED : 0)
                | (array.hasCharacteristics(DISTINCT) ? Spliterator.DISTINCT : 0);
    }

    /**
     * Returns spliterator for specified array.
     *
     * @param array array to create spliterator for.
     * @param <E> element type.
     * @return spliterator for specified array.
     */
    @SuppressWarnings("unchecked")
    static <E> Spliterator<E> spliterator(@Nonnull Array<E> array) {
        if (array instanceof IntArray) {
            return (Spliterator <E>) spliterator((IntArray) array);
        }
        if (array instanceof LongArray) {
            return (Spliterator <E>) spliterator((LongArray) array);
        }
        if (array instanceof DoubleArray) {
            return (Spliterator <E>) spliterator((DoubleArray) array);
        }
        if (array instanceof ObjectArray) {
            return spliterator((ObjectArray<E>) array);
        }
        return Spliterators.spliterator(array.iterator(), array.length(), getCharacteristics(array));
    }
    
    /**
     * Returns default primitive spliterator based on internal array if array implements {@code UnsafeArray},
     * otherwise based on array's iterator.
     *
     * @param array array to create spliterator for.
     * @return spliterator for specified array.
     */
    static Spliterator.OfInt spliterator(@Nonnull IntArray array) {
        if (array instanceof UnsafeArray) {
            int offset = array.offset();
            int length = array.length();
            return Spliterators.spliterator(UnsafeArray.intReadAccess(array), offset, offset + length, getCharacteristics(array));
        }
        return Spliterators.spliterator(array.iterator(), array.length(), getCharacteristics(array));
    }
    
    /**
     * Returns default primitive spliterator based on internal array if array implements {@code UnsafeArray},
     * otherwise based on array's iterator.
     *
     * @param array array to create spliterator for.
     * @return spliterator for specified array.
     */
    static Spliterator.OfLong spliterator(@Nonnull LongArray array) {
        if (array instanceof UnsafeArray) {
            int offset = array.offset();
            int length = array.length();
            return Spliterators.spliterator(UnsafeArray.longReadAccess(array), offset, offset + length, getCharacteristics(array));
        }
        return Spliterators.spliterator(array.iterator(), array.length(), getCharacteristics(array));
    }

    /**
     * Returns default primitive spliterator based on internal array if array implements {@code UnsafeArray},
     * otherwise based on array's iterator.
     *
     * @param array array to create spliterator for.
     * @return spliterator for specified array.
     */
    static Spliterator.OfDouble spliterator(@Nonnull DoubleArray array) {
        if (array instanceof UnsafeArray) {
            int offset = array.offset();
            int length = array.length();
            return Spliterators.spliterator(UnsafeArray.doubleReadAccess(array), offset, offset + length, getCharacteristics(array));
        }
        return Spliterators.spliterator(array.iterator(), array.length(), getCharacteristics(array));
    }
    
    /**
     * Returns default primitive spliterator based on internal array if array implements {@code UnsafeArray},
     * otherwise based on array's iterator.
     *
     * @param array array to create spliterator for.
     * @return spliterator for specified array.
     */
    private static <E> Spliterator<E> spliterator(@Nonnull ObjectArray<E> array) {
        int offset = array.offset();
        int length = array.length();
        if (UnsafeArray.hasDirectAccess(array, Object.class)) {
            Object[] a = UnsafeArray.readAccess(array);
            if (array.hasCharacteristics(SORTED)) {
                return new DelegatingSpliteratorImpl<>(Spliterators.spliterator(a, offset, offset + length,
                        getCharacteristics(array)), array.getComparator());

            } else {
                return Spliterators.spliterator(a, offset, offset + length, getCharacteristics(array));
            }
        } else {
            if (array.hasCharacteristics(SORTED)) {
                return new DelegatingSpliteratorImpl<>(Spliterators.spliterator(array.iterator(), length,
                        getCharacteristics(array)), array.getComparator());

            } else {
                return Spliterators.spliterator(array.iterator(), length, getCharacteristics(array));
            }
        }
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class DelegatingSpliteratorImpl<E> implements Spliterator<E> {

        final Spliterator<E> delegate;
        final Comparator<? super E> comparator;

        DelegatingSpliteratorImpl(Spliterator<E> delegate, Comparator<? super E> comparator) {
            this.delegate = delegate;
            this.comparator = comparator;
        }

        @Override
        public Comparator<? super E> getComparator() {
            return comparator;
        }

        @Override
        public boolean hasCharacteristics(int characteristics) {
            return delegate.hasCharacteristics(characteristics);
        }

        @Override
        public int characteristics() {
            return delegate.characteristics();
        }

        @Override
        public long getExactSizeIfKnown() {
            return delegate.getExactSizeIfKnown();
        }

        @Override
        public long estimateSize() {
            return delegate.estimateSize();
        }

        @Override
        public Spliterator<E> trySplit() {
            return new DelegatingSpliteratorImpl<>(delegate.trySplit(), comparator);
        }

        @Override
        public void forEachRemaining(Consumer<? super E> action) {
            delegate.forEachRemaining(action);
        }

        @Override
        public boolean tryAdvance(Consumer<? super E> action) {
            return delegate.tryAdvance(action);
        }
    }

    /**
     * Returns a sequential {@code Stream} with this array as its source.
     *
     * @param array array to create stream for.
     * @return a sequential {@code Stream} over the elements in this array.
     */
    static <E> Stream<E> stream(Array<E> array) {
        if (array.hasCharacteristics(MUTABLE)) {
            int characteristics = getCharacteristics(array);
            Supplier <Spliterator<E>> supplier = () -> {
                if (getCharacteristics(array) != characteristics) {
                    throw new ConcurrentModificationException("Spliterator characteristics modified.");
                }
                return array.spliterator();
            };
            return StreamSupport.stream(supplier, characteristics);
        }
        return StreamSupport.stream(array.spliterator());
    }

    /**
     * Returns a parallel {@code Stream} with this array as its source.
     *
     * @param array array to create stream for.
     * @return a parallel {@code Stream} over the elements in this array.
     */
    static <E> Stream<E> parallelStream(Array<E> array) {
        if (array.hasCharacteristics(MUTABLE)) {
            int characteristics = getCharacteristics(array);
            Supplier <Spliterator<E>> supplier = () -> {
                if (getCharacteristics(array) != characteristics) {
                    throw new ConcurrentModificationException("Spliterator characteristics modified.");
                }
                return array.spliterator();
            };
            return StreamSupport.parallelStream(supplier, characteristics);
        }
        return StreamSupport.parallelStream(array.spliterator());
    }

    /**
     * Returns a sequential {@code IntStream} with this array as its source.
     *
     * @param array array to create stream for.
     * @return a sequential {@code IntStream} over the elements in this array.
     */
    static IntStream intStream(IntArray array) {
        if (array.hasCharacteristics(MUTABLE)) {
            int characteristics = getCharacteristics(array);
            Supplier <Spliterator.OfInt> supplier = () -> {
                if (getCharacteristics(array) != characteristics) {
                    throw new ConcurrentModificationException("Spliterator characteristics modified.");
                }
                return array.spliterator();
            };
            return StreamSupport.intStream(supplier, characteristics);
        }
        return StreamSupport.intStream(array.spliterator());
    }

    /**
     * Returns a parallel {@code IntStream} with this array as its source.
     *
     * @param array array to create stream for.
     * @return a parallel {@code IntStream} over the elements in this array.
     */
    static IntStream intParallelStream(IntArray array) {
        if (array.hasCharacteristics(MUTABLE)) {
            int characteristics = getCharacteristics(array);
            Supplier <Spliterator.OfInt> supplier = () -> {
                if (getCharacteristics(array) != characteristics) {
                    throw new ConcurrentModificationException("Spliterator characteristics modified.");
                }
                return array.spliterator();
            };
            return StreamSupport.intParallelStream(supplier, characteristics);
        }
        return StreamSupport.intParallelStream(array.spliterator());
    }

    /**
     * Returns a sequential {@code LongStream} with this array as its source.
     *
     * @param array array to create stream for.
     * @return a sequential {@code LongStream} over the elements in this array.
     */
    static LongStream longStream(LongArray array) {
        if (array.hasCharacteristics(MUTABLE)) {
            int characteristics = getCharacteristics(array);
            Supplier <Spliterator.OfLong> supplier = () -> {
                if (getCharacteristics(array) != characteristics) {
                    throw new ConcurrentModificationException("Spliterator characteristics modified.");
                }
                return array.spliterator();
            };
            return StreamSupport.longStream(supplier, characteristics);
        }
        return StreamSupport.longStream(array.spliterator());
    }

    /**
     * Returns a parallel {@code LongStream} with this array as its source.
     *
     * @param array array to create stream for.
     * @return a parallel {@code LongStream} over the elements in this array.
     */
    static LongStream longParallelStream(LongArray array) {
        if (array.hasCharacteristics(MUTABLE)) {
            int characteristics = getCharacteristics(array);
            Supplier <Spliterator.OfLong> supplier = () -> {
                if (getCharacteristics(array) != characteristics) {
                    throw new ConcurrentModificationException("Spliterator characteristics modified.");
                }
                return array.spliterator();
            };
            return StreamSupport.longParallelStream(supplier, characteristics);
        }
        return StreamSupport.longParallelStream(array.spliterator());
    }

    /**
     * Returns a sequential {@code DoubleStream} with this array as its source.
     *
     * @param array array to create stream for.
     * @return a sequential {@code DoubleStream} over the elements in this array.
     */
    static DoubleStream doubleStream(DoubleArray array) {
        if (array.hasCharacteristics(MUTABLE)) {
            int characteristics = getCharacteristics(array);
            Supplier <Spliterator.OfDouble> supplier = () -> {
                if (getCharacteristics(array) != characteristics) {
                    throw new ConcurrentModificationException("Spliterator characteristics modified.");
                }
                return array.spliterator();
            };
            return StreamSupport.doubleStream(supplier, characteristics);
        }
        return StreamSupport.doubleStream(array.spliterator());
    }

    /**
     * Returns a parallel {@code DoubleStream} with this array as its source.
     *
     * @param array array to create stream for.
     * @return a parallel {@code DoubleStream} over the elements in this array.
     */
    static DoubleStream doubleParallelStream(DoubleArray array) {
        if (array.hasCharacteristics(MUTABLE)) {
            int characteristics = getCharacteristics(array);
            Supplier <Spliterator.OfDouble> supplier = () -> {
                if (getCharacteristics(array) != characteristics) {
                    throw new ConcurrentModificationException("Spliterator characteristics modified.");
                }
                return array.spliterator();
            };
            return StreamSupport.doubleParallelStream(supplier, characteristics);
        }
        return StreamSupport.doubleParallelStream(array.spliterator());
    }

    /**
     * Searches the specified array for the specified value using the binary search algorithm. The array must be sorted
     * prior to making this call. If it is not sorted, the results are undefined. If the array contains multiple
     * elements equal to the specified key, there is no guarantee which one will be found.
     *
     * <p>Unlike binarySearch, this method does not throw an exception if specified array is not marked as sorted.</p>
     *
     * @param array the array to be searched.
     * @param key the value to be searched for.
     * @return index of the search key, if it is contained in the array; otherwise,
     * <tt>(-(<i>insertion point</i>) - 1)</tt>. The <i>insertion point</i> is defined as the point at which the key
     * would be inserted into the array: the index of the first element greater than the key, or {@code array.length()}
     * if all elements in the array are less than the specified key. Note that this guarantees that the return value
     * will be {@literal >= 0} if and only if the key is found.
     */
    public static int binarySearch(@Nonnull BooleanArray array, boolean key) {
        boolean[] a = UnsafeArray.booleanReadAccess(array);
        int offset = array.offset();
        int low = offset;
        int high = offset + array.length() - 1;
        while (low <= high) {
            int mid = (low + high) >>> 1;
            boolean midVal = a[mid];
            if (midVal == key) {
                return mid - offset;
            } else if (!midVal) {
                low = mid + 1;
            } else {
                high = mid - 1;
            }
        }
        return -(low + 1) - offset;  // key not found.
    }

    /**
     * Searches the specified array for the specified value using the binary search algorithm. The array must be sorted
     * prior to making this call. If it is not sorted, the results are undefined. If the array contains multiple
     * elements equal to the specified key, there is no guarantee which one will be found.
     *
     * <p>Unlike binarySearch, this method does not throw an exception if specified array is not marked as sorted.</p>
     *
     * @param array the array to be searched.
     * @param key the value to be searched for.
     * @return index of the search key, if it is contained in the array; otherwise,
     * <tt>(-(<i>insertion point</i>) - 1)</tt>. The <i>insertion point</i> is defined as the point at which the key
     * would be inserted into the array: the index of the first element greater than the key, or {@code array.length()}
     * if all elements in the array are less than the specified key. Note that this guarantees that the return value
     * will be {@literal >= 0} if and only if the key is found.
     */
    public static int binarySearch(@Nonnull ByteArray array, byte key) {
        byte[] a = UnsafeArray.byteReadAccess(array);
        int offset = array.offset();
        int low = offset;
        int high = offset + array.length() - 1;
        while (low <= high) {
            int mid = (low + high) >>> 1;
            byte midVal = a[mid];
            if (midVal < key) {
                low = mid + 1;
            } else if (midVal > key) {
                high = mid - 1;
            } else {
                return mid - offset;
            }
        }
        return -(low + 1) - offset;  // key not found.
    }

    /**
     * Searches the specified array for the specified value using the binary search algorithm. The array must be sorted
     * prior to making this call. If it is not sorted, the results are undefined. If the array contains multiple
     * elements equal to the specified key, there is no guarantee which one will be found.
     *
     * <p>Unlike binarySearch, this method does not throw an exception if specified array is not marked as sorted.</p>
     *
     * @param array the array to be searched.
     * @param key the value to be searched for.
     * @return index of the search key, if it is contained in the array; otherwise,
     * <tt>(-(<i>insertion point</i>) - 1)</tt>. The <i>insertion point</i> is defined as the point at which the key
     * would be inserted into the array: the index of the first element greater than the key, or {@code array.length()}
     * if all elements in the array are less than the specified key. Note that this guarantees that the return value
     * will be {@literal >= 0} if and only if the key is found.
     */
    public static int binarySearch(@Nonnull CharArray array, char key) {
        char[] a = UnsafeArray.charReadAccess(array);
        int offset = array.offset();
        int low = offset;
        int high = offset + array.length() - 1;
        while (low <= high) {
            int mid = (low + high) >>> 1;
            char midVal = a[mid];
            if (midVal < key) {
                low = mid + 1;
            } else if (midVal > key) {
                high = mid - 1;
            } else {
                return mid - offset;
            }
        }
        return -(low + 1) - offset;  // key not found.
    }

    /**
     * Searches the specified array for the specified value using the binary search algorithm. The array must be sorted
     * prior to making this call. If it is not sorted, the results are undefined. If the array contains multiple
     * elements equal to the specified key, there is no guarantee which one will be found.
     *
     * <p>Unlike binarySearch, this method does not throw an exception if specified array is not marked as sorted.</p>
     *
     * @param array the array to be searched.
     * @param key the value to be searched for.
     * @return index of the search key, if it is contained in the array; otherwise,
     * <tt>(-(<i>insertion point</i>) - 1)</tt>. The <i>insertion point</i> is defined as the point at which the key
     * would be inserted into the array: the index of the first element greater than the key, or {@code array.length()}
     * if all elements in the array are less than the specified key. Note that this guarantees that the return value
     * will be {@literal >= 0} if and only if the key is found.
     */
    public static int binarySearch(@Nonnull ShortArray array, short key) {
        short[] a = UnsafeArray.shortReadAccess(array);
        int offset = array.offset();
        int low = offset;
        int high = offset + array.length() - 1;
        while (low <= high) {
            int mid = (low + high) >>> 1;
            short midVal = a[mid];
            if (midVal < key) {
                low = mid + 1;
            } else if (midVal > key) {
                high = mid - 1;
            } else {
                return mid - offset;
            }
        }
        return -(low + 1) - offset;  // key not found.
    }

    /**
     * Searches the specified array for the specified value using the binary search algorithm. The array must be sorted
     * prior to making this call. If it is not sorted, the results are undefined. If the array contains multiple
     * elements equal to the specified key, there is no guarantee which one will be found.
     *
     * <p>Unlike binarySearch, this method does not throw an exception if specified array is not marked as sorted.</p>
     *
     * @param array the array to be searched.
     * @param key the value to be searched for.
     * @return index of the search key, if it is contained in the array; otherwise,
     * <tt>(-(<i>insertion point</i>) - 1)</tt>. The <i>insertion point</i> is defined as the point at which the key
     * would be inserted into the array: the index of the first element greater than the key, or {@code array.length()}
     * if all elements in the array are less than the specified key. Note that this guarantees that the return value
     * will be {@literal >= 0} if and only if the key is found.
     */
    public static int binarySearch(@Nonnull IntArray array, int key) {
        int[] a = UnsafeArray.intReadAccess(array);
        int offset = array.offset();
        int low = offset;
        int high = offset + array.length() - 1;
        while (low <= high) {
            int mid = (low + high) >>> 1;
            int midVal = a[mid];
            if (midVal < key) {
                low = mid + 1;
            } else if (midVal > key) {
                high = mid - 1;
            } else {
                return mid - offset;
            }
        }
        return -(low + 1) - offset;  // key not found.
    }

    /**
     * Searches the specified array for the specified value using the binary search algorithm. The array must be sorted
     * prior to making this call. If it is not sorted, the results are undefined. If the array contains multiple
     * elements equal to the specified key, there is no guarantee which one will be found.
     *
     * <p>Unlike binarySearch, this method does not throw an exception if specified array is not marked as sorted.</p>
     *
     * @param array the array to be searched.
     * @param key the value to be searched for.
     * @return index of the search key, if it is contained in the array; otherwise,
     * <tt>(-(<i>insertion point</i>) - 1)</tt>. The <i>insertion point</i> is defined as the point at which the key
     * would be inserted into the array: the index of the first element greater than the key, or {@code array.length()}
     * if all elements in the array are less than the specified key. Note that this guarantees that the return value
     * will be {@literal >= 0} if and only if the key is found.
     */
    public static int binarySearch(@Nonnull LongArray array, long key) {
        long[] a = UnsafeArray.longReadAccess(array);
        int offset = array.offset();
        int low = offset;
        int high = offset + array.length() - 1;
        while (low <= high) {
            int mid = (low + high) >>> 1;
            long midVal = a[mid];
            if (midVal < key) {
                low = mid + 1;
            } else if (midVal > key) {
                high = mid - 1;
            } else {
                return mid - offset;
            }
        }
        return -(low + 1) - offset;  // key not found.
    }

    /**
     * Searches the specified array for the specified value using the binary search algorithm. The array must be sorted
     * prior to making this call. If it is not sorted, the results are undefined. If the array contains multiple
     * elements equal to the specified key, there is no guarantee which one will be found.
     *
     * <p>Unlike binarySearch, this method does not throw an exception if specified array is not marked as sorted.</p>
     *
     * @param array the array to be searched.
     * @param key the value to be searched for.
     * @return index of the search key, if it is contained in the array; otherwise,
     * <tt>(-(<i>insertion point</i>) - 1)</tt>. The <i>insertion point</i> is defined as the point at which the key
     * would be inserted into the array: the index of the first element greater than the key, or {@code array.length()}
     * if all elements in the array are less than the specified key. Note that this guarantees that the return value
     * will be {@literal >= 0} if and only if the key is found.
     */
    public static int binarySearch(@Nonnull FloatArray array, float key) {
        float[] a = UnsafeArray.floatReadAccess(array);
        int offset = array.offset();
        int low = offset;
        int high = offset + array.length() - 1;
        while (low <= high) {
            int mid = (low + high) >>> 1;
            float midVal = a[mid];
            if (midVal < key) {
                low = mid + 1;
            } else if (midVal > key) {
                high = mid - 1;
            } else {
                return mid - offset;
            }
        }
        return -(low + 1) - offset;  // key not found.
    }

    /**
     * Searches the specified array for the specified value using the binary search algorithm. The array must be sorted
     * prior to making this call. If it is not sorted, the results are undefined. If the array contains multiple
     * elements equal to the specified key, there is no guarantee which one will be found.
     *
     * <p>Unlike binarySearch, this method does not throw an exception if specified array is not marked as sorted.</p>
     *
     * @param array the array to be searched.
     * @param key the value to be searched for.
     * @return index of the search key, if it is contained in the array; otherwise,
     * <tt>(-(<i>insertion point</i>) - 1)</tt>. The <i>insertion point</i> is defined as the point at which the key
     * would be inserted into the array: the index of the first element greater than the key, or {@code array.length()}
     * if all elements in the array are less than the specified key. Note that this guarantees that the return value
     * will be {@literal >= 0} if and only if the key is found.
     */
    public static int binarySearch(@Nonnull DoubleArray array, double key) {
        double[] a = UnsafeArray.doubleReadAccess(array);
        int offset = array.offset();
        int low = offset;
        int high = offset + array.length() - 1;
        while (low <= high) {
            int mid = (low + high) >>> 1;
            double midVal = a[mid];
            if (midVal < key) {
                low = mid + 1;
            } else if (midVal > key) {
                high = mid - 1;
            } else {
                return mid - offset;
            }
        }
        return -(low + 1) - offset;  // key not found.
    }

    /**
     * Searches the specified array for the specified object using the binary search algorithm. The array must be sorted
     * into ascending order according to natural ordering the specified comparator (as by the
     * {@link java.util.Arrays#sort(Object[], Comparator) sort(T[], Comparator)} method) prior to making this call. If
     * it is not sorted, the results are undefined. If the array contains multiple elements equal to the specified
     * object, there is no guarantee which one will be found.
     *
     * <p>Unlike binarySearch, this method does not throw an exception if specified array is not marked as sorted.</p>
     *
     * @param array the array to be searched.
     * @param key the value to be searched for.
     * @return index of the search key, if it is contained in the array; otherwise,
     * <tt>(-(<i>insertion point</i>) - 1)</tt>. The <i>insertion point</i> is defined as the point at which the key
     * would be inserted into the array: the index of the first element greater than the key, or {@code array.length()}
     * if all elements in the array are less than the specified key. Note that this guarantees that the return value
     * will be {@literal >= 0} if and only if the key is found.
     * @throws NullPointerException if key is null.
     * @throws ClassCastException if key is not comparable or if key is not compatible with array's elements.
     */
    public static int binarySearch(@Nonnull ObjectArray<?> array, Object key) {
        int length = array.length();
        if (length == 0) {
            if (key == null) {
                throw new NullPointerException();
            }
        }
        Comparable k = (Comparable) key;
        Object[] a = UnsafeArray.readAccess(array);
        int offset = array.offset();
        int low = offset;
        int high = offset + length - 1;
        while (low <= high) {
            int mid = (low + high) >>> 1;
            @SuppressWarnings("rawtypes")
            Comparable midVal = (Comparable) a[mid];
            @SuppressWarnings("unchecked")
            int cmp = k.compareTo(midVal);
            if (cmp > 0) {
                low = mid + 1;
            } else if (cmp < 0) {
                high = mid - 1;
            } else {
                return mid - offset; // key found
            }
        }
        return -(low + 1) - offset;  // key not found.
    }

    /**
     * Searches the specified array for the specified object using the binary search algorithm. The array must be sorted
     * into ascending order according to the specified comparator (as by the
     * {@link java.util.Arrays#sort(Object[], Comparator) sort(T[], Comparator)} method) prior to making this call.
     * If it is not sorted, the results are undefined. If the array contains multiple elements equal to the specified
     * object, there is no guarantee which one will be found.
     *
     * <p>Unlike binarySearch, this method does not throw an exception if specified array is not marked as sorted.</p>
     *
     * @param <E> element type.
     * @param array the array to be searched.
     * @param key the value to be searched for.
     * @param c the comparator by which the array is ordered.  A
     *        <tt>null</tt> value indicates that the elements'
     *        <i>Comparable natural ordering</i> should be used.
     * @return index of the search key, if it is contained in the array;
     *         otherwise, <tt>(-(<i>insertion point</i>) - 1)</tt>.  The
     *         <i>insertion point</i> is defined as the point at which the
     *         key would be inserted into the array: the index of the first
     *         element greater than the key, or <tt>a.length()</tt> if all
     *         elements in the array are less than the specified key.  Note
     *         that this guarantees that the return value will be &gt;= 0 if
     *         and only if the key is found.
     * @throws NullPointerException if key is null and specified comparator not compatible with null values.
     * @throws ClassCastException if key is not comparable to the elements of the array using the specified comparator.
     */
    public static <E> int binarySearch(@Nonnull ObjectArray<E> array, E key, Comparator<? super E> c) {
        if (c == null) {
            return binarySearch(array, key);
        }
        int length = array.length();
        if (length == 0) {  // Type (and possibly null) check.
            c.compare(key, key);
        }
        @SuppressWarnings("unchecked")
        E[] a = (E[]) UnsafeArray.readAccess(array);
        int offset = array.offset();
        int low = offset;
        int high = offset + array.length() - 1;
        while (low <= high) {
            int mid = (low + high) >>> 1;
            E midVal = a[mid];
            int cmp = c.compare(key, midVal);
            if (cmp > 0) {
                low = mid + 1;
            } else if (cmp < 0) {
                high = mid - 1;
            } else {
                return mid - offset; // Key found.
            }
        }
        return -(low + 1) - offset;  // Key not found.
    }

    /**
     * Searches a range of the specified array for the lowest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for (inclusive).
     * @return lowest index for the value found at {@code index}.
     */
    static int binarySearchFirst(@Nonnull BooleanArray array, int index) {
        rangeCheckFirst(array.length(), index);
        if (index == 0) {
            return index;
        }
        boolean[] a = UnsafeArray.booleanReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        boolean key = a[mid];
        int low = offset;
        int high = mid--;
        do {
            if (a[mid] == key) {
                int diff = ((high - mid) << 1);
                high = mid;
                mid = max(low, high - diff);
            } else {
                low = mid + 1;
                mid = high - 1;
            }
        } while (high > low);
        return high - offset;
    }

    /**
     * Searches a range of the specified array for the highest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for.
     * @return highest index for the value found at {@code index}.
     */
    static int binarySearchLast(@Nonnull BooleanArray array, int index) {
        rangeCheckLast(array.length(), index);
        if (index == array.offset() + array.length() - 1) {
            return index;
        }
        boolean[] a = UnsafeArray.booleanReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        boolean key = a[mid];
        int low = mid++;
        int high = offset + array.length();
        while (low < high - 1) {  // Start looking from mid (in search for key) and double distance each time.
            if (a[mid] == key) {
                int diff = ((mid - low) << 1);
                low = mid;
                mid = min(high - 1, low + diff);
            } else {
                high = mid;
                mid = low + 1;
            }
        }
        return low - offset;
    }

    /**
     * Searches a range of the specified array for the lowest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for (inclusive).
     * @return lowest index for the value found at {@code index}.
     */
    static int binarySearchFirst(@Nonnull ByteArray array, int index) {
        rangeCheckFirst(array.length(), index);
        if (index == 0) {
            return index;
        }
        byte[] a = UnsafeArray.byteReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        byte key = a[mid];
        int low = offset;
        int high = mid--;
        do {
            if (a[mid] < key) {
                low = mid + 1;
                mid = high - 1;
            } else {
                int diff = ((high - mid) << 1);
                high = mid;
                mid = max(low, high - diff);
            }
        } while (high > low);
        return high - offset;
    }

    /**
     * Searches a range of the specified array for the highest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for.
     * @return highest index for the value found at {@code index}.
     */
    static int binarySearchLast(@Nonnull ByteArray array, int index) {
        rangeCheckLast(array.length(), index);
        if (index == array.offset() + array.length() - 1) {
            return index;
        }
        byte[] a = UnsafeArray.byteReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        byte key = a[mid];
        int low = mid++;
        int high = offset + array.length();
        while (low < high - 1) {  // Start looking from mid (in search for key) and double distance each time.
            if (a[mid] > key) {
                high = mid;
                mid = low + 1;
            } else {
                int diff = ((mid - low) << 1);
                low = mid;
                mid = min(high - 1, low + diff);
            }
        }
        return low - offset;
    }

    /**
     * Searches a range of the specified array for the lowest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for (inclusive).
     * @return lowest index for the value found at {@code index}.
     */
    static int binarySearchFirst(@Nonnull CharArray array, int index) {
        rangeCheckFirst(array.length(), index);
        if (index == 0) {
            return index;
        }
        char[] a = UnsafeArray.charReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        char key = a[mid];
        int low = offset;
        int high = mid--;
        do {
            if (a[mid] < key) {
                low = mid + 1;
                mid = high - 1;
            } else {
                int diff = ((high - mid) << 1);
                high = mid;
                mid = max(low, high - diff);
            }
        } while (high > low);
        return high - offset;
    }

    /**
     * Searches a range of the specified array for the highest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for.
     * @return highest index for the value found at {@code index}.
     */
    static int binarySearchLast(@Nonnull CharArray array, int index) {
        rangeCheckLast(array.length(), index);
        if (index == array.offset() + array.length() - 1) {
            return index;
        }
        char[] a = UnsafeArray.charReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        char key = a[mid];
        int low = mid++;
        int high = offset + array.length();
        while (low < high - 1) {  // Start looking from mid (in search for key) and double distance each time.
            if (a[mid] > key) {
                high = mid;
                mid = low + 1;
            } else {
                int diff = ((mid - low) << 1);
                low = mid;
                mid = min(high - 1, low + diff);
            }
        }
        return low - offset;
    }

    /**
     * Searches a range of the specified array for the lowest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for (inclusive).
     * @return lowest index for the value found at {@code index}.
     */
    static int binarySearchFirst(@Nonnull ShortArray array, int index) {
        rangeCheckFirst(array.length(), index);
        if (index == 0) {
            return index;
        }
        short[] a = UnsafeArray.shortReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        short key = a[mid];
        int low = offset;
        int high = mid--;
        do {
            if (a[mid] < key) {
                low = mid + 1;
                mid = high - 1;
            } else {
                int diff = ((high - mid) << 1);
                high = mid;
                mid = max(low, high - diff);
            }
        } while (high > low);
        return high - offset;
    }

    /**
     * Searches a range of the specified array for the highest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for.
     * @return highest index for the value found at {@code index}.
     */
    static int binarySearchLast(@Nonnull ShortArray array, int index) {
        rangeCheckLast(array.length(), index);
        if (index == array.offset() + array.length() - 1) {
            return index;
        }
        short[] a = UnsafeArray.shortReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        short key = a[mid];
        int low = mid++;
        int high = offset + array.length();
        while (low < high - 1) {  // Start looking from mid (in search for key) and double distance each time.
            if (a[mid] > key) {
                high = mid;
                mid = low + 1;
            } else {
                int diff = ((mid - low) << 1);
                low = mid;
                mid = min(high - 1, low + diff);
            }
        }
        return low - offset;
    }

    /**
     * Searches a range of the specified array for the lowest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for (inclusive).
     * @return lowest index for the value found at {@code index}.
     */
    static int binarySearchFirst(@Nonnull IntArray array, int index) {
        rangeCheckFirst(array.length(), index);
        if (index == 0) {
            return index;
        }
        int[] a = UnsafeArray.intReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        int key = a[mid];
        int low = offset;
        int high = mid--;
        do {
            if (a[mid] < key) {
                low = mid + 1;
                mid = high - 1;
            } else {
                int diff = ((high - mid) << 1);
                high = mid;
                mid = max(low, high - diff);
            }
        } while (high > low);
        return high - offset;
    }

    /**
     * Searches a range of the specified array for the highest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for.
     * @return highest index for the value found at {@code index}.
     */
    static int binarySearchLast(@Nonnull IntArray array, int index) {
        rangeCheckLast(array.length(), index);
        if (index == array.offset() + array.length() - 1) {
            return index;
        }
        int[] a = UnsafeArray.intReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        int key = a[mid];
        int low = mid++;
        int high = offset + array.length();
        while (low < high - 1) {  // Start looking from mid (in search for key) and double distance each time.
            if (a[mid] > key) {
                high = mid;
                mid = low + 1;
            } else {
                int diff = ((mid - low) << 1);
                low = mid;
                mid = min(high - 1, low + diff);
            }
        }
        return low - offset;
    }

    /**
     * Searches a range of the specified array for the lowest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for (inclusive).
     * @return lowest index for the value found at {@code index}.
     */
    static int binarySearchFirst(@Nonnull LongArray array, int index) {
        rangeCheckFirst(array.length(), index);
        if (index == 0) {
            return index;
        }
        long[] a = UnsafeArray.longReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        long key = a[mid];
        int low = offset;
        int high = mid--;
        do {
            if (a[mid] < key) {
                low = mid + 1;
                mid = high - 1;
            } else {
                int diff = ((high - mid) << 1);
                high = mid;
                mid = max(low, high - diff);
            }
        } while (high > low);
        return high - offset;
    }

    /**
     * Searches a range of the specified array for the highest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for.
     * @return highest index for the value found at {@code index}.
     */
    static int binarySearchLast(@Nonnull LongArray array, int index) {
        rangeCheckLast(array.length(), index);
        if (index == array.offset() + array.length() - 1) {
            return index;
        }
        long[] a = UnsafeArray.longReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        long key = a[mid];
        int low = mid++;
        int high = offset + array.length();
        while (low < high - 1) {  // Start looking from mid (in search for key) and double distance each time.
            if (a[mid] > key) {
                high = mid;
                mid = low + 1;
            } else {
                int diff = ((mid - low) << 1);
                low = mid;
                mid = min(high - 1, low + diff);
            }
        }
        return low - offset;
    }

    /**
     * Searches a range of the specified array for the lowest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for (inclusive).
     * @return lowest index for the value found at {@code index}.
     */
    static int binarySearchFirst(@Nonnull FloatArray array, int index) {
        rangeCheckFirst(array.length(), index);
        if (index == 0) {
            return index;
        }
        float[] a = UnsafeArray.floatReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        float key = a[mid];
        int low = offset;
        int high = mid--;
        do {
            if (a[mid] < key) {
                low = mid + 1;
                mid = high - 1;
            } else {
                int diff = ((high - mid) << 1);
                high = mid;
                mid = max(low, high - diff);
            }
        } while (high > low);
        return high - offset;
    }

    /**
     * Searches a range of the specified array for the highest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for.
     * @return highest index for the value found at {@code index}.
     */
    static int binarySearchLast(@Nonnull FloatArray array, int index) {
        rangeCheckLast(array.length(), index);
        if (index == array.offset() + array.length() - 1) {
            return index;
        }
        float[] a = UnsafeArray.floatReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        float key = a[mid];
        int low = mid++;
        int high = offset + array.length();
        while (low < high - 1) {  // Start looking from mid (in search for key) and double distance each time.
            if (a[mid] > key) {
                high = mid;
                mid = low + 1;
            } else {
                int diff = ((mid - low) << 1);
                low = mid;
                mid = min(high - 1, low + diff);
            }
        }
        return low - offset;
    }

    /**
     * Searches a range of the specified array for the lowest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for (inclusive).
     * @return lowest index for the value found at {@code index}.
     */
    static int binarySearchFirst(@Nonnull DoubleArray array, int index) {
        rangeCheckFirst(array.length(), index);
        if (index == 0) {
            return index;
        }
        double[] a = UnsafeArray.doubleReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        double key = a[mid];
        int low = offset;
        int high = mid--;
        do {
            if (a[mid] < key) {
                low = mid + 1;
                mid = high - 1;
            } else {
                int diff = ((high - mid) << 1);
                high = mid;
                mid = max(low, high - diff);
            }
        } while (high > low);
        return high - offset;
    }

    /**
     * Searches a range of the specified array for the highest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for.
     * @return highest index for the value found at {@code index}.
     */
    static int binarySearchLast(@Nonnull DoubleArray array, int index) {
        rangeCheckLast(array.length(), index);
        if (index == array.offset() + array.length() - 1) {
            return index;
        }
        double[] a = UnsafeArray.doubleReadAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        double key = a[mid];
        int low = mid++;
        int high = offset + array.length();
        while (low < high - 1) {  // Start looking from mid (in search for key) and double distance each time.
            if (a[mid] > key) {
                high = mid;
                mid = low + 1;
            } else {
                int diff = ((mid - low) << 1);
                low = mid;
                mid = min(high - 1, low + diff);
            }
        }
        return low - offset;
    }

    /**
     * Searches a range of the specified array for the lowest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for (inclusive).
     * @return lowest index for the value found at {@code index}.
     * @throws NullPointerException if array contains null elements.
     * @throws ClassCastException if array contains non {@code Comparable} elements.
     */
    static int binarySearchFirst(@Nonnull ObjectArray<?> array, int index) {
        rangeCheckFirst(array.length(), index);
        if (index == 0) {
            return index;
        }
        Object[] a = UnsafeArray.readAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        Comparable key = (Comparable) a[mid];
        int low = offset;
        int high = mid--;
        do {
            @SuppressWarnings("rawtypes")
            Comparable midVal = (Comparable) a[mid];
            @SuppressWarnings("unchecked")
            int cmp = key.compareTo(midVal);
            if (cmp > 0) {
                low = mid + 1;
                mid = high - 1;
            } else {
                int diff = ((high - mid) << 1);
                high = mid;
                mid = max(low, high - diff);
            }
        } while (high > low);
        return high - offset;
    }

    /**
     * Searches a range of the specified array for the lowest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param <E> element type.
     * @param array the array to be searched.
     * @param index the index of the value that is searched for (inclusive).
     * @param c the comparator by which the array is ordered.  A
     *        <tt>null</tt> value indicates that the elements'
     *        <i>Comparable natural ordering</i> should be used.
     * @return lowest index for the value found at {@code index}.
     * @throws NullPointerException if array contains null elements and specified comparator not compatible with null values.
     */
    @SuppressWarnings("unchecked")
    static <E> int binarySearchFirst(@Nonnull ObjectArray<E> array, int index, @Nonnull Comparator<? super E> c) {
        rangeCheckFirst(array.length(), index);
        if (index == 0) {
            return index;
        }
        Object[] a = UnsafeArray.readAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        E key = (E) a[mid];
        int low = offset;
        int high = mid--;
        do {
            if (c.compare(key, (E) a[mid]) > 0) {
                low = mid + 1;
                mid = high - 1;
            } else {
                int diff = ((high - mid) << 1);
                high = mid;
                mid = max(low, high - diff);
            }
        } while (high > low);
        return high - offset;
    }

    /**
     * Searches a range of the specified array for the highest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param array the array to be searched
     * @param index the index of the value that is searched for.
     * @return highest index for the value found at {@code index}.
     * @throws NullPointerException if array contains null elements.
     * @throws ClassCastException if array contains non {@code Comparable} elements.
     */
    static int binarySearchLast(@Nonnull ObjectArray<?> array, int index) {
        rangeCheckLast(array.length(), index);
        if (index == array.offset() + array.length() - 1) {
            return index;
        }
        Object[] a = UnsafeArray.readAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        Comparable key = (Comparable) a[mid];
        int low = mid++;
        int high = offset + array.length();
        while (low < high - 1) {  // Start looking from mid (in search for key) and double distance each time.
            @SuppressWarnings("rawtypes")
            Comparable midVal = (Comparable) a[mid];
            @SuppressWarnings("unchecked")
            int cmp = key.compareTo(midVal);
            if (cmp < 0) {
                high = mid;
                mid = low + 1;
            } else {
                int diff = ((mid - low) << 1);
                low = mid;
                mid = min(high - 1, low + diff);
            }
        }
        return low - offset;
    }

    /**
     * Searches a range of the specified array for the highest index for the value found at {@code index}, using the
     * binary search algorithm. The range must be sorted prior to making this call. If it is not sorted, the results
     * are undefined.
     *
     * @param <E> element type.
     * @param array the array to be searched
     * @param index the index of the value that is searched for.
     * @param c the comparator by which the array is ordered.  A
     *        <tt>null</tt> value indicates that the elements'
     *        <i>Comparable natural ordering</i> should be used.
     * @return highest index for the value found at {@code index}.
     * @throws NullPointerException if array contains null elements and specified comparator not compatible with null values.
     */
    @SuppressWarnings("unchecked")
    static <E> int binarySearchLast(@Nonnull ObjectArray<E> array, int index, @Nonnull Comparator<? super E> c) {
        rangeCheckLast(array.length(), index);
        if (index == array.offset() + array.length() - 1) {
            return index;
        }
        Object[] a = UnsafeArray.readAccess(array);
        int offset = array.offset();
        int mid = offset + index;
        E key = (E) a[mid];
        int low = mid++;
        int high = offset + array.length();
        while (low < high - 1) {  // Start looking from mid (in search for key) and double distance each time.
            if (c.compare(key, (E) a[mid]) < 0) {
                high = mid;
                mid = low + 1;
            } else {
                int diff = ((mid - low) << 1);
                low = mid;
                mid = min(high - 1, low + diff);
            }
        }
        return low - offset;
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  BooleanArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (boolean value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static MutableBooleanArray synchronizedArray(@Nonnull MutableBooleanArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedMutableBooleanArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  BooleanArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (boolean value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static MutableBooleanArray synchronizedArray(@Nonnull MutableBooleanArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedMutableBooleanArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  ByteArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (byte value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static MutableByteArray synchronizedArray(@Nonnull MutableByteArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedMutableByteArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  ByteArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (byte value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static MutableByteArray synchronizedArray(@Nonnull MutableByteArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedMutableByteArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  CharArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (char value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static MutableCharArray synchronizedArray(@Nonnull MutableCharArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedMutableCharArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  CharArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (char value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static MutableCharArray synchronizedArray(@Nonnull MutableCharArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedMutableCharArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  ShortArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (short value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static MutableShortArray synchronizedArray(@Nonnull MutableShortArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedMutableShortArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  ShortArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (short value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static MutableShortArray synchronizedArray(@Nonnull MutableShortArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedMutableShortArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  IntArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (int value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static MutableIntArray synchronizedArray(@Nonnull MutableIntArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedMutableIntArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  IntArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (int value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static MutableIntArray synchronizedArray(@Nonnull MutableIntArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedMutableIntArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  LongArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (long value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static MutableLongArray synchronizedArray(@Nonnull MutableLongArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedMutableLongArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  LongArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (long value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static MutableLongArray synchronizedArray(@Nonnull MutableLongArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedMutableLongArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  FloatArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (float value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static MutableFloatArray synchronizedArray(@Nonnull MutableFloatArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedMutableFloatArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  FloatArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (float value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static MutableFloatArray synchronizedArray(@Nonnull MutableFloatArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedMutableFloatArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  DoubleArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (double value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static MutableDoubleArray synchronizedArray(@Nonnull MutableDoubleArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedMutableDoubleArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  DoubleArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (double value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static MutableDoubleArray synchronizedArray(@Nonnull MutableDoubleArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedMutableDoubleArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  {@literal ObjectArray<Object>} a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (Object value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param <E> element type.
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static <E> MutableObjectArray<E> synchronizedArray(@Nonnull MutableObjectArray<E> array) {
        Objects.requireNonNull(array);
        return new SynchronizedMutableObjectArray<>(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  {@literal ObjectArray<Object>} a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (Object value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param <E> element type.
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static <E> MutableObjectArray<E> synchronizedArray(@Nonnull MutableObjectArray<E> array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedMutableObjectArray<>(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  BooleanArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (boolean value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static DynamicBooleanArray synchronizedArray(@Nonnull DynamicBooleanArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedDynamicBooleanArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  BooleanArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (boolean value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static DynamicBooleanArray synchronizedArray(@Nonnull DynamicBooleanArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedDynamicBooleanArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  ByteArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (byte value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static DynamicByteArray synchronizedArray(@Nonnull DynamicByteArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedDynamicByteArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  ByteArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (byte value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static DynamicByteArray synchronizedArray(@Nonnull DynamicByteArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedDynamicByteArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  CharArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (char value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static DynamicCharArray synchronizedArray(@Nonnull DynamicCharArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedDynamicCharArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  CharArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (char value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static DynamicCharArray synchronizedArray(@Nonnull DynamicCharArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedDynamicCharArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  ShortArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (short value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static DynamicShortArray synchronizedArray(@Nonnull DynamicShortArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedDynamicShortArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  ShortArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (short value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static DynamicShortArray synchronizedArray(@Nonnull DynamicShortArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedDynamicShortArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  IntArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (int value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static DynamicIntArray synchronizedArray(@Nonnull DynamicIntArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedDynamicIntArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  IntArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (int value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static DynamicIntArray synchronizedArray(@Nonnull DynamicIntArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedDynamicIntArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  LongArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (long value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static DynamicLongArray synchronizedArray(@Nonnull DynamicLongArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedDynamicLongArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  LongArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (long value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static DynamicLongArray synchronizedArray(@Nonnull DynamicLongArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedDynamicLongArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  FloatArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (float value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static DynamicFloatArray synchronizedArray(@Nonnull DynamicFloatArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedDynamicFloatArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  FloatArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (float value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static DynamicFloatArray synchronizedArray(@Nonnull DynamicFloatArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedDynamicFloatArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  DoubleArray a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (double value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static DynamicDoubleArray synchronizedArray(@Nonnull DynamicDoubleArray array) {
        Objects.requireNonNull(array);
        return new SynchronizedDynamicDoubleArray(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  DoubleArray a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (double value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static DynamicDoubleArray synchronizedArray(@Nonnull DynamicDoubleArray array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedDynamicDoubleArray(array, mutex);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the returned
     * array when iterating over it:
     * <pre>
     *  {@literal ObjectArray<Object>} a = Arrays.synchronizedArray(myArray);
     *     ...
     *  synchronized (a) {
     *      for (Object value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param <E> element type.
     * @param array the array to be "wrapped" in a synchronized array.
     * @return a synchronized view of the specified array.
     */
    public static <E> DynamicObjectArray<E> synchronizedArray(@Nonnull DynamicObjectArray<E> array) {
        Objects.requireNonNull(array);
        return new SynchronizedDynamicObjectArray<>(array);
    }

    /**
     * Returns a synchronized (thread-safe) array backed by the specified
     * {@code array}. In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing array is accomplished
     * through the returned array.<p>
     *
     * It is imperative that the user manually synchronize on the mutex
     * array when iterating over it:
     * <pre>
     *  Object mutex = new Object();
     *  {@literal ObjectArray<Object>} a = Arrays.synchronizedArray(myArray, mutex);
     *     ...
     *  synchronized (mutex) {
     *      for (Object value : a) {
     *         ...
     *      }
     *  }
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * @param <E> element type.
     * @param array the array to be "wrapped" in a synchronized array.
     * @param mutex mutex used by the synchronized array to synchronize access.
     * @return a synchronized view of the specified array.
     */
    public static <E> DynamicObjectArray<E> synchronizedArray(@Nonnull DynamicObjectArray<E> array, @Nonnull Object mutex) {
        Objects.requireNonNull(array);
        Objects.requireNonNull(mutex);
        return new SynchronizedDynamicObjectArray<>(array, mutex);
    }

    /**
     * Sorts the range of {@code array}.
     *
     * @param array the array to be sorted.
     * @param fromIndex the index of the first element, inclusive, to be sorted
     * @param toIndex the index of the last element, exclusive, to be sorted
     */
    static void sort(@Nonnull boolean[] array, int fromIndex, int toIndex) {
        int trueCount = 0;
        for (int i = fromIndex; i < toIndex; i++) {
            if (array[i]) {
                trueCount++;
            }
        }
        for (int i = fromIndex; i < toIndex - trueCount; i++) {
            array[i] = false;
        }
        for (int i = fromIndex + trueCount; i < toIndex; i++) {
            array[i] = true;
        }
    }

    /**
     * Sorts the range of {@code array}. The underlying implementation utilizes multiple threads to reduce
     * sorting time. The algorithm requires a working space equal to the size of the to be sorted range.
     *
     * @param array the array to be sorted.
     * @param fromIndex the index of the first element, inclusive, to be sorted
     * @param toIndex the index of the last element, exclusive, to be sorted
     */
    static void parallelSort(@Nonnull boolean[] array, int fromIndex, int toIndex) {
        // TODO Add parallel implementation.
        sort(array, fromIndex, toIndex);
    }

    /**
     * Reverses the order of the elements in the specified array.
     *
     * <p>This method runs in linear time.</p>
     *
     * @param array the array whose elements are to be reversed.
     */
    public static void reverse(@Nonnull MutableBooleanArray array) {
        int length = array.length();
        for (int i = 0, mid = length >> 1, j = length - 1; i < mid; i++, j--) {
            swap(array, i, j);
        }
    }

    /**
     * Reverses the order of the elements in the specified array.
     *
     * <p>This method runs in linear time.</p>
     *
     * @param array the array whose elements are to be reversed.
     */
    public static void reverse(@Nonnull MutableByteArray array) {
        int length = array.length();
        for (int i = 0, mid = length >> 1, j = length - 1; i < mid; i++, j--) {
            swap(array, i, j);
        }
    }

    /**
     * Reverses the order of the elements in the specified array.
     *
     * <p>This method runs in linear time.</p>
     *
     * @param array the array whose elements are to be reversed.
     */
    public static void reverse(@Nonnull MutableCharArray array) {
        int length = array.length();
        for (int i = 0, mid = length >> 1, j = length - 1; i < mid; i++, j--) {
            swap(array, i, j);
        }
    }

    /**
     * Reverses the order of the elements in the specified array.
     *
     * <p>This method runs in linear time.</p>
     *
     * @param array the array whose elements are to be reversed.
     */
    public static void reverse(@Nonnull MutableShortArray array) {
        int length = array.length();
        for (int i = 0, mid = length >> 1, j = length - 1; i < mid; i++, j--) {
            swap(array, i, j);
        }
    }

    /**
     * Reverses the order of the elements in the specified array.
     *
     * <p>This method runs in linear time.</p>
     *
     * @param array the array whose elements are to be reversed.
     */
    public static void reverse(@Nonnull MutableIntArray array) {
        int length = array.length();
        for (int i = 0, mid = length >> 1, j = length - 1; i < mid; i++, j--) {
            swap(array, i, j);
        }
    }

    /**
     * Reverses the order of the elements in the specified array.
     *
     * <p>This method runs in linear time.</p>
     *
     * @param array the array whose elements are to be reversed.
     */
    public static void reverse(@Nonnull MutableLongArray array) {
        int length = array.length();
        for (int i = 0, mid = length >> 1, j = length - 1; i < mid; i++, j--) {
            swap(array, i, j);
        }
    }

    /**
     * Reverses the order of the elements in the specified array.
     *
     * <p>This method runs in linear time.</p>
     *
     * @param array the array whose elements are to be reversed.
     */
    public static void reverse(@Nonnull MutableFloatArray array) {
        int length = array.length();
        for (int i = 0, mid = length >> 1, j = length - 1; i < mid; i++, j--) {
            swap(array, i, j);
        }
    }

    /**
     * Reverses the order of the elements in the specified array.
     *
     * <p>This method runs in linear time.</p>
     *
     * @param array the array whose elements are to be reversed.
     */
    public static void reverse(@Nonnull MutableDoubleArray array) {
        int length = array.length();
        for (int i = 0, mid = length >> 1, j = length - 1; i < mid; i++, j--) {
            swap(array, i, j);
        }
    }

    /**
     * Reverses the order of the elements in the specified array.
     *
     * <p>This method runs in linear time.</p>
     *
     * @param array the array whose elements are to be reversed.
     */
    @SuppressWarnings({"rawtypes", "unchecked"})
    public static void reverse(@Nonnull MutableObjectArray<?> array) {
        int length = array.length();
        for (int i = 0, mid = length >> 1, j = length - 1; i < mid; i++, j--) {
            swap(array, i, j);
        }
    }

    /**
     * Swaps the values at the specified positions in the specified array.
     * (If the specified positions are equal, invoking this method leaves
     * the array unchanged.)
     *
     * @param array the array in which to swap elements.
     * @param i the index of one value to be swapped.
     * @param j the index of the other value to be swapped.
     * @throws IndexOutOfBoundsException if either <tt>i</tt> or <tt>j</tt>
     *         is out of range (i &lt; 0 || i &gt;= array.length()
     *         || j &lt; 0 || j &gt;= array.length()).
     */
    public static void swap(@Nonnull MutableBooleanArray array, int i, int j) {
        array.set(i, array.set(j, array.get(i)));
    }

    /**
     * Swaps the values at the specified positions in the specified array.
     * (If the specified positions are equal, invoking this method leaves
     * the array unchanged.)
     *
     * @param array the array in which to swap elements.
     * @param i the index of one value to be swapped.
     * @param j the index of the other value to be swapped.
     * @throws IndexOutOfBoundsException if either <tt>i</tt> or <tt>j</tt>
     *         is out of range (i &lt; 0 || i &gt;= array.length()
     *         || j &lt; 0 || j &gt;= array.length()).
     */
    public static void swap(@Nonnull MutableByteArray array, int i, int j) {
        array.set(i, array.set(j, array.get(i)));
    }

    /**
     * Swaps the values at the specified positions in the specified array.
     * (If the specified positions are equal, invoking this method leaves
     * the array unchanged.)
     *
     * @param array the array in which to swap elements.
     * @param i the index of one value to be swapped.
     * @param j the index of the other value to be swapped.
     * @throws IndexOutOfBoundsException if either <tt>i</tt> or <tt>j</tt>
     *         is out of range (i &lt; 0 || i &gt;= array.length()
     *         || j &lt; 0 || j &gt;= array.length()).
     */
    public static void swap(@Nonnull MutableCharArray array, int i, int j) {
        array.set(i, array.set(j, array.get(i)));
    }

    /**
     * Swaps the values at the specified positions in the specified array.
     * (If the specified positions are equal, invoking this method leaves
     * the array unchanged.)
     *
     * @param array the array in which to swap elements.
     * @param i the index of one value to be swapped.
     * @param j the index of the other value to be swapped.
     * @throws IndexOutOfBoundsException if either <tt>i</tt> or <tt>j</tt>
     *         is out of range (i &lt; 0 || i &gt;= array.length()
     *         || j &lt; 0 || j &gt;= array.length()).
     */
    public static void swap(@Nonnull MutableShortArray array, int i, int j) {
        array.set(i, array.set(j, array.get(i)));
    }

    /**
     * Swaps the values at the specified positions in the specified array.
     * (If the specified positions are equal, invoking this method leaves
     * the array unchanged.)
     *
     * @param array the array in which to swap elements.
     * @param i the index of one value to be swapped.
     * @param j the index of the other value to be swapped.
     * @throws IndexOutOfBoundsException if either <tt>i</tt> or <tt>j</tt>
     *         is out of range (i &lt; 0 || i &gt;= array.length()
     *         || j &lt; 0 || j &gt;= array.length()).
     */
    public static void swap(@Nonnull MutableIntArray array, int i, int j) {
        array.set(i, array.set(j, array.get(i)));
    }

    /**
     * Swaps the values at the specified positions in the specified array.
     * (If the specified positions are equal, invoking this method leaves
     * the array unchanged.)
     *
     * @param array the array in which to swap elements.
     * @param i the index of one value to be swapped.
     * @param j the index of the other value to be swapped.
     * @throws IndexOutOfBoundsException if either <tt>i</tt> or <tt>j</tt>
     *         is out of range (i &lt; 0 || i &gt;= array.length()
     *         || j &lt; 0 || j &gt;= array.length()).
     */
    public static void swap(@Nonnull MutableLongArray array, int i, int j) {
        array.set(i, array.set(j, array.get(i)));
    }

    /**
     * Swaps the values at the specified positions in the specified array.
     * (If the specified positions are equal, invoking this method leaves
     * the array unchanged.)
     *
     * @param array the array in which to swap elements.
     * @param i the index of one value to be swapped.
     * @param j the index of the other value to be swapped.
     * @throws IndexOutOfBoundsException if either <tt>i</tt> or <tt>j</tt>
     *         is out of range (i &lt; 0 || i &gt;= array.length()
     *         || j &lt; 0 || j &gt;= array.length()).
     */
    public static void swap(@Nonnull MutableFloatArray array, int i, int j) {
        array.set(i, array.set(j, array.get(i)));
    }

    /**
     * Swaps the values at the specified positions in the specified array.
     * (If the specified positions are equal, invoking this method leaves
     * the array unchanged.)
     *
     * @param array the array in which to swap elements.
     * @param i the index of one value to be swapped.
     * @param j the index of the other value to be swapped.
     * @throws IndexOutOfBoundsException if either <tt>i</tt> or <tt>j</tt>
     *         is out of range (i &lt; 0 || i &gt;= array.length()
     *         || j &lt; 0 || j &gt;= array.length()).
     */
    public static void swap(@Nonnull MutableDoubleArray array, int i, int j) {
        array.set(i, array.set(j, array.get(i)));
    }

    /**
     * Swaps the elements at the specified positions in the specified array.
     * (If the specified positions are equal, invoking this method leaves
     * the array unchanged.)
     *
     * @param array the array in which to swap elements.
     * @param i the index of one elements to be swapped.
     * @param j the index of the other elements to be swapped.
     * @throws IndexOutOfBoundsException if either <tt>i</tt> or <tt>j</tt>
     *         is out of range (i &lt; 0 || i &gt;= array.length()
     *         || j &lt; 0 || j &gt;= array.length()).
     */
    @SuppressWarnings({"rawtypes", "unchecked"})
    public static void swap(@Nonnull MutableObjectArray<?> array, int i, int j) {
        //noinspection UnnecessaryLocalVariable
        final MutableObjectArray a = array;
        a.set(i, a.set(j, a.get(i)));
    }

    /**
     * Assigns the specified boolean value to each element of the {@code array}.
     *
     * @param array the array to be filled.
     * @param value the value to be stored in all elements of the array's range.
     */
    public static void fill(@Nonnull MutableBooleanArray array, boolean value) {
        array.replaceAllBoolean(ignore -> value);
    }

    /**
     * Assigns the specified byte value to each element of the {@code array}.
     *
     * @param array the array to be filled.
     * @param value the value to be stored in all elements of the array's range.
     */
    public static void fill(@Nonnull MutableByteArray array, byte value) {
        array.replaceAllByte(ignore -> value);
    }

    /**
     * Assigns the specified char value to each element of the {@code array}.
     *
     * @param array the array to be filled.
     * @param value the value to be stored in all elements of the array's range.
     */
    public static void fill(@Nonnull MutableCharArray array, char value) {
        array.replaceAllChar(ignore -> value);
    }

    /**
     * Assigns the specified short value to each element of the {@code array}.
     *
     * @param array the array to be filled.
     * @param value the value to be stored in all elements of the array's range.
     */
    public static void fill(@Nonnull MutableShortArray array, short value) {
        array.replaceAllShort(ignore -> value);
    }

    /**
     * Assigns the specified int value to each element of the {@code array}.
     *
     * @param array the array to be filled.
     * @param value the value to be stored in all elements of the array's range.
     */
    public static void fill(@Nonnull MutableIntArray array, int value) {
        array.replaceAllInt(ignore -> value);
    }

    /**
     * Assigns the specified long value to each element of the {@code array}.
     *
     * @param array the array to be filled.
     * @param value the value to be stored in all elements of the array's range.
     */
    public static void fill(@Nonnull MutableLongArray array, long value) {
        array.replaceAllLong(ignore -> value);
    }

    /**
     * Assigns the specified float value to each element of the {@code array}.
     *
     * @param array the array to be filled.
     * @param value the value to be stored in all elements of the array's range.
     */
    public static void fill(@Nonnull MutableFloatArray array, float value) {
        array.replaceAllFloat(ignore -> value);
    }

    /**
     * Assigns the specified double value to each element of the {@code array}.
     *
     * @param array the array to be filled.
     * @param value the value to be stored in all elements of the array's range.
     */
    public static void fill(@Nonnull MutableDoubleArray array, double value) {
        array.replaceAllDouble(ignore -> value);
    }

    /**
     * Assigns the specified object to each element of the {@code array}.
     *
     * @param <E> element type.
     * @param array the array to be filled.
     * @param value the value to be stored in all elements of the array's range.
     */
    public static <E> void fill(@Nonnull MutableObjectArray<E> array, E value) {
        array.replaceAll(ignore -> value);
    }

    /**
     * Assigns the objects returned {@code supplier} to the elements of the {@code array}. An object is requested from
     * {@code supplier} for each element that is updated.
     *
     * @param <E> element type.
     * @param array the array to be filled.
     * @param supplier factory for objects to be stored in elements of the array's range.
     */
    public static <E> void fillBySupplier(@Nonnull MutableObjectArray<E> array, @Nonnull Supplier<E> supplier) {
        array.replaceAll(ignore -> supplier.get());
    }

    /**
     * Assigns the objects returned {@code supplier} to the elements of the {@code array}. An object is requested from
     * {@code supplier} for each element that is updated.
     *
     * @param <E> element type.
     * @param array the array to be filled.
     * @param function factory for objects to be stored in elements of the array's range.
     */
    public static <E> void fillByFunction(@Nonnull MutableObjectArray<E> array, @Nonnull IntFunction<E> function) {
        ArrayIterator<E> it = array.iterator();
        int length = array.length();
        for (int i = 0; i < length; i++) {
            it.next();
            it.set(function.apply(i));
        }
    }

    /**
     * Returns an immutable array consisting of {@code n} copies of the specified boolean value. The newly allocated
     * boolean array is tiny (it only contains the boolean value). This method is useful in combination with the
     * {@link DynamicArray#addAll} or {@link MutableArray#setAll} methods. The returned array is serializable.
     *
     * @param  n the number of elements in the returned array.
     * @param  value the value to appear repeatedly in the returned array.
     * @return an immutable array of {@code n} copies of the specified value.
     * @throws IllegalArgumentException if {@code n < 0}.
     */
    public static BooleanArray nElements(int n, boolean value) {
        if (n < 0) {
            throw new IllegalArgumentException(String.valueOf(n));
        }
        return new ElementsBooleanArray(n, value);
    }

    /**
     * Returns an immutable array consisting of {@code n} copies of the specified byte value. The newly allocated byte
     * array is tiny (it only contains the byte value). This method is useful in combination with the
     * {@link DynamicArray#addAll} or {@link MutableArray#setAll} methods. The returned array is serializable.
     *
     * @param  n the number of elements in the returned array.
     * @param  value the value to appear repeatedly in the returned array.
     * @return an immutable array of {@code n} copies of the specified value.
     * @throws IllegalArgumentException if {@code n < 0}.
     */
    public static ByteArray nElements(int n, byte value) {
        if (n < 0) {
            throw new IllegalArgumentException(String.valueOf(n));
        }
        return new ElementsByteArray(n, value);
    }

    /**
     * Returns an immutable array consisting of {@code n} copies of the specified char value. The newly allocated char
     * array is tiny (it only contains the char value). This method is useful in combination with the
     * {@link DynamicArray#addAll} or {@link MutableArray#setAll} methods. The returned array is serializable.
     *
     * @param  n the number of elements in the returned array.
     * @param  value the value to appear repeatedly in the returned array.
     * @return an immutable array of {@code n} copies of the specified value.
     * @throws IllegalArgumentException if {@code n < 0}.
     */
    public static CharArray nElements(int n, char value) {
        if (n < 0) {
            throw new IllegalArgumentException(String.valueOf(n));
        }
        return new ElementsCharArray(n, value);
    }

    /**
     * Returns an immutable array consisting of {@code n} copies of the specified short value. The newly allocated short
     * array is tiny (it only contains the short value). This method is useful in combination with the
     * {@link DynamicArray#addAll} or {@link MutableArray#setAll} methods. The returned array is serializable.
     *
     * @param  n the number of elements in the returned array.
     * @param  value the value to appear repeatedly in the returned array.
     * @return an immutable array of {@code n} copies of the specified value.
     * @throws IllegalArgumentException if {@code n < 0}.
     */
    public static ShortArray nElements(int n, short value) {
        if (n < 0) {
            throw new IllegalArgumentException(String.valueOf(n));
        }
        return new ElementsShortArray(n, value);
    }

    /**
     * Returns an immutable array consisting of {@code n} copies of the specified int value. The newly allocated int
     * array is tiny (it only contains the int value). This method is useful in combination with the
     * {@link DynamicArray#addAll} or {@link MutableArray#setAll} methods. The returned array is serializable.
     *
     * @param  n the number of elements in the returned array.
     * @param  value the value to appear repeatedly in the returned array.
     * @return an immutable array of {@code n} copies of the specified value.
     * @throws IllegalArgumentException if {@code n < 0}.
     */
    public static IntArray nElements(int n, int value) {
        if (n < 0) {
            throw new IllegalArgumentException(String.valueOf(n));
        }
        return new ElementsIntArray(n, value);
    }

    /**
     * Returns an immutable array consisting of {@code n} copies of the specified long value. The newly allocated long
     * array is tiny (it only contains the long value). This method is useful in combination with the
     * {@link DynamicArray#addAll} or {@link MutableArray#setAll} methods. The returned array is serializable.
     *
     * @param  n the number of elements in the returned array.
     * @param  value the value to appear repeatedly in the returned array.
     * @return an immutable array of {@code n} copies of the specified value.
     * @throws IllegalArgumentException if {@code n < 0}.
     */
    public static LongArray nElements(int n, long value) {
        if (n < 0) {
            throw new IllegalArgumentException(String.valueOf(n));
        }
        return new ElementsLongArray(n, value);
    }

    /**
     * Returns an immutable array consisting of {@code n} copies of the specified float value. The newly allocated float
     * array is tiny (it only contains the float value). This method is useful in combination with the
     * {@link DynamicArray#addAll} or {@link MutableArray#setAll} methods. The returned array is serializable.
     *
     * @param  n the number of elements in the returned array.
     * @param  value the value to appear repeatedly in the returned array.
     * @return an immutable array of {@code n} copies of the specified value.
     * @throws IllegalArgumentException if {@code n < 0}.
     */
    public static FloatArray nElements(int n, float value) {
        if (n < 0) {
            throw new IllegalArgumentException(String.valueOf(n));
        }
        return new ElementsFloatArray(n, value);
    }

    /**
     * Returns an immutable array consisting of {@code n} copies of the specified double value. The newly allocated
     * double array is tiny (it only contains the double value). This method is useful in combination with the
     * {@link DynamicArray#addAll} or {@link MutableArray#setAll} methods. The returned array is serializable.
     *
     * @param  n the number of elements in the returned list.
     * @param  value the value to appear repeatedly in the returned array.
     * @return an immutable array of {@code n} copies of the specified value.
     * @throws IllegalArgumentException if {@code n < 0}.
     */
    public static DoubleArray nElements(int n, double value) {
        if (n < 0) {
            throw new IllegalArgumentException(String.valueOf(n));
        }
        return new ElementsDoubleArray(n, value);
    }

    /**
     * Returns an immutable array consisting of {@code n} copies of the specified object reference. The newly
     * allocated object array is tiny (it only contains the object reference). This method is useful in combination with
     * the {@link DynamicArray#addAll} or {@link MutableArray#setAll} methods. The returned array is serializable.
     *
     * @param <E> element type.
     * @param  n the number of elements in the returned array.
     * @param  value the object reference to appear repeatedly in the returned array.
     * @return an immutable array of {@code n} copies of the specified object reference.
     * @throws IllegalArgumentException if {@code n < 0}.
     */
    @SuppressWarnings("unchecked")
    public static <E> ObjectArray<E> nElements(int n, @Nullable E value) {
        if (n < 0) {
            throw new IllegalArgumentException(String.valueOf(n));
        }
        return new ElementsObjectArray<>(n, value);
    }

    /**
     * Returns an immutable array filled with {@code n} elements obtained from {@code supplier}.
     *
     * @param <E> element type.
     * @param n the number of elements in the returned array.
     * @param supplier factory for objects to be stored in elements of the array's range.
     * @return an immutable array of {@code n} elements supplied by {@code supplier}.
     * @throws IllegalArgumentException if {@code n < 0}.
     */
    @SuppressWarnings("unchecked")
    public static <E> ObjectArray<E> nElementsBySupplier(int n, @Nonnull Supplier<E> supplier) {
        if (n < 0) {
            throw new IllegalArgumentException(String.valueOf(n));
        }
        Objects.requireNonNull(supplier);
        Object[] a = new Object[n];
        for (int i = 0; i < n; i++) {
            a[i] = supplier.get();
        }
        return ObjectArray.unsafeValueOf((E[]) a);
    }

    /**
     * Returns an immutable array filled with {@code n} elements obtained from {@code supplier}.
     *
     * @param <E> element type.
     * @param n the number of elements in the returned array.
     * @param function factory for objects to be stored in elements of the array's range.
     * @return an immutable array of {@code n} elements supplied by {@code supplier}.
     * @throws IllegalArgumentException if {@code n < 0}.
     */
    @SuppressWarnings("unchecked")
    public static <E> ObjectArray<E> nElementsByFunction(int n, @Nonnull IntFunction<E> function) {
        if (n < 0) {
            throw new IllegalArgumentException(String.valueOf(n));
        }
        Objects.requireNonNull(function);
        Object[] a = new Object[n];
        for (int i = 0; i < n; i++) {
            a[i] = function.apply(i);
        }
        return ObjectArray.unsafeValueOf((E[]) a);
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedMutableBooleanArray implements MutableBooleanArray, UnsafeArray<Boolean>, Serializable {
        final MutableBooleanArray array;
        final Object mutex;

        SynchronizedMutableBooleanArray(MutableBooleanArray array) {
            this.array = array;
            this.mutex = this;
        }

        SynchronizedMutableBooleanArray(MutableBooleanArray array, Object mutex) {
            this.array = array;
            this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public boolean[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Boolean> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public Comparator<? super Boolean> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Boolean, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Boolean get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public boolean getBoolean(int index) {
           synchronized (mutex) { return array.getBoolean(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsBoolean(boolean value) {
            synchronized (mutex) { return array.containsBoolean(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfBoolean(boolean value) {
            synchronized (mutex) { return array.indexOfBoolean(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfBoolean(boolean value) {
            synchronized (mutex) { return array.lastIndexOfBoolean(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Boolean> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachBoolean(@Nonnull BooleanConsumer consumer) {
            synchronized (mutex) { array.forEachBoolean(consumer); }
        }

        @Override public Boolean set(int index, Boolean value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public boolean setBoolean(int index, boolean value) {
           synchronized (mutex) { return array.setBoolean(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Boolean> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }        

        @Override public void replaceAll(@Nonnull UnaryOperator<Boolean> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public void replaceAllBoolean(@Nonnull BooleanUnaryOperator operator) {
            synchronized (mutex) { array.replaceAllBoolean(operator); }
        }

        @Override public boolean[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public boolean[] toArray(@Nonnull boolean[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public MutableBooleanArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedMutableBooleanArray(array.offset(offset), mutex); }
        }

        @Override public MutableBooleanArray length(int length) {
           synchronized (mutex) { return new SynchronizedMutableBooleanArray(array.length(length), mutex); }
        }

        @Override public MutableBooleanArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedMutableBooleanArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override public ArrayIterator.OfBoolean iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfBoolean iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator<Boolean> spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public Stream<Boolean> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Boolean> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            static final long serialVersionUID = -7159301299825603217L;

            final MutableBooleanArray array;
            final Object mutex;

            SerializationProxy(SynchronizedMutableBooleanArray array) {
                this.array = array.array;
                this.mutex = array.mutex;
            }

            private Object readResolve() {
                return new SynchronizedMutableBooleanArray(array, mutex);
            }

            private void writeObject(ObjectOutputStream s) throws IOException {
                synchronized (mutex) { s.defaultWriteObject(); }
            }
        }
    }    
    
    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedMutableByteArray implements MutableByteArray, UnsafeArray<Byte>, Serializable {
        final MutableByteArray array;
        final Object mutex;

        SynchronizedMutableByteArray(MutableByteArray array) {
            this.array = array;
            this.mutex = this;
        }

        SynchronizedMutableByteArray(MutableByteArray array, Object mutex) {
            this.array = array;
            this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public byte[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Byte> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Byte, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super Byte> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }
        
        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Byte get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public byte getByte(int index) {
           synchronized (mutex) { return array.getByte(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsByte(byte value) {
            synchronized (mutex) { return array.containsByte(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfByte(byte value) {
            synchronized (mutex) { return array.indexOfByte(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfByte(byte value) {
            synchronized (mutex) { return array.lastIndexOfByte(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Byte> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachByte(@Nonnull ByteConsumer consumer) {
            synchronized (mutex) { array.forEachByte(consumer); }
        }

        @Override public Byte set(int index, Byte value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public byte setByte(int index, byte value) {
           synchronized (mutex) { return array.setByte(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Byte> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }        

        @Override public void replaceAll(@Nonnull UnaryOperator<Byte> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public void replaceAllByte(@Nonnull ByteUnaryOperator operator) {
            synchronized (mutex) { array.replaceAllByte(operator); }
        }

        @Override public byte[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public byte[] toArray(@Nonnull byte[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public MutableByteArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedMutableByteArray(array.offset(offset), mutex); }
        }

        @Override public MutableByteArray length(int length) {
           synchronized (mutex) { return new SynchronizedMutableByteArray(array.length(length), mutex); }
        }

        @Override public MutableByteArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedMutableByteArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override public ArrayIterator.OfByte iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfByte iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator<Byte> spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public Stream<Byte> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Byte> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            static final long serialVersionUID = 7108555788317605319L;

            final MutableByteArray array;
            final Object mutex;

            SerializationProxy(SynchronizedMutableByteArray array) {
                this.array = array.array;
                this.mutex = array.mutex;
            }

            private Object readResolve() {
                return new SynchronizedMutableByteArray(array, mutex);
            }

            private void writeObject(ObjectOutputStream s) throws IOException {
                synchronized (mutex) { s.defaultWriteObject(); }
            }
        }
    }
    
    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedMutableCharArray implements MutableCharArray, UnsafeArray<Character>, Serializable {
        final MutableCharArray array;
        final Object mutex;

        SynchronizedMutableCharArray(MutableCharArray array) {
            this.array = array;
            this.mutex = this;
        }

        SynchronizedMutableCharArray(MutableCharArray array, Object mutex) {
            this.array = array;
            this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public char[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Character> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Character, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super Character> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Character get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public char getChar(int index) {
           synchronized (mutex) { return array.getChar(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsChar(char value) {
            synchronized (mutex) { return array.containsChar(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfChar(char value) {
            synchronized (mutex) { return array.indexOfChar(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfChar(char value) {
            synchronized (mutex) { return array.lastIndexOfChar(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Character> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachChar(@Nonnull CharConsumer consumer) {
            synchronized (mutex) { array.forEachChar(consumer); }
        }

        @Override public Character set(int index, Character value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public char setChar(int index, char value) {
           synchronized (mutex) { return array.setChar(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Character> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }        

        @Override public void replaceAll(@Nonnull UnaryOperator<Character> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public void replaceAllChar(@Nonnull CharUnaryOperator operator) {
            synchronized (mutex) { array.replaceAllChar(operator); }
        }

        @Override public char[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public char[] toArray(@Nonnull char[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public MutableCharArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedMutableCharArray(array.offset(offset), mutex); }
        }

        @Override public MutableCharArray length(int length) {
           synchronized (mutex) { return new SynchronizedMutableCharArray(array.length(length), mutex); }
        }

        @Override public MutableCharArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedMutableCharArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override public ArrayIterator.OfChar iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfChar iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator<Character> spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public Stream<Character> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Character> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            static final long serialVersionUID = 6471348135383260060L;

            final MutableCharArray array;
            final Object mutex;

            SerializationProxy(SynchronizedMutableCharArray array) {
                this.array = array.array;
                this.mutex = array.mutex;
            }

            private Object readResolve() {
                return new SynchronizedMutableCharArray(array, mutex);
            }

            private void writeObject(ObjectOutputStream s) throws IOException {
                synchronized (mutex) { s.defaultWriteObject(); }
            }
        }
    }    
    
    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedMutableShortArray implements MutableShortArray, UnsafeArray<Short>, Serializable {
        final MutableShortArray array;
        final Object mutex;

        SynchronizedMutableShortArray(MutableShortArray array) {
            this.array = array;
            this.mutex = this;
        }

        SynchronizedMutableShortArray(MutableShortArray array, Object mutex) {
            this.array = array;
            this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public short[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Short> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Short, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super Short> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Short get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public short getShort(int index) {
           synchronized (mutex) { return array.getShort(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsShort(short value) {
            synchronized (mutex) { return array.containsShort(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfShort(short value) {
            synchronized (mutex) { return array.indexOfShort(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfShort(short value) {
            synchronized (mutex) { return array.lastIndexOfShort(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Short> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachShort(@Nonnull ShortConsumer consumer) {
            synchronized (mutex) { array.forEachShort(consumer); }
        }

        @Override public Short set(int index, Short value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public short setShort(int index, short value) {
           synchronized (mutex) { return array.setShort(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Short> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }        

        @Override public void replaceAll(@Nonnull UnaryOperator<Short> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public void replaceAllShort(@Nonnull ShortUnaryOperator operator) {
            synchronized (mutex) { array.replaceAllShort(operator); }
        }

        @Override public short[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public short[] toArray(@Nonnull short[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public MutableShortArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedMutableShortArray(array.offset(offset), mutex); }
        }

        @Override public MutableShortArray length(int length) {
           synchronized (mutex) { return new SynchronizedMutableShortArray(array.length(length), mutex); }
        }

        @Override public MutableShortArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedMutableShortArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override public ArrayIterator.OfShort iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfShort iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator<Short> spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public Stream<Short> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Short> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            static final long serialVersionUID = 4637786014035490545L;

            final MutableShortArray array;
            final Object mutex;

            SerializationProxy(SynchronizedMutableShortArray array) {
                this.array = array.array;
                this.mutex = array.mutex;
            }

            private Object readResolve() {
                return new SynchronizedMutableShortArray(array, mutex);
            }

            private void writeObject(ObjectOutputStream s) throws IOException {
                synchronized (mutex) { s.defaultWriteObject(); }
            }
        }
    }    
    
    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedMutableIntArray implements MutableIntArray, UnsafeArray<Integer>, Serializable {
        final MutableIntArray array;
        final Object mutex;

        SynchronizedMutableIntArray(MutableIntArray array) {
            this.array = array;
            this.mutex = this;
        }

        SynchronizedMutableIntArray(MutableIntArray array, Object mutex) {
            this.array = array;
            this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public int[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Integer> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Integer, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super Integer> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Integer get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public int getInt(int index) {
           synchronized (mutex) { return array.getInt(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsInt(int value) {
            synchronized (mutex) { return array.containsInt(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfInt(int value) {
            synchronized (mutex) { return array.indexOfInt(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfInt(int value) {
            synchronized (mutex) { return array.lastIndexOfInt(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Integer> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachInt(@Nonnull IntConsumer consumer) {
            synchronized (mutex) { array.forEachInt(consumer); }
        }

        @Override public Integer set(int index, Integer value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public int setInt(int index, int value) {
           synchronized (mutex) { return array.setInt(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Integer> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }        

        @Override public void replaceAll(@Nonnull UnaryOperator<Integer> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public void replaceAllInt(@Nonnull IntUnaryOperator operator) {
            synchronized (mutex) { array.replaceAllInt(operator); }
        }

        @Override public int[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public int[] toArray(@Nonnull int[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public MutableIntArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedMutableIntArray(array.offset(offset), mutex); }
        }

        @Override public MutableIntArray length(int length) {
           synchronized (mutex) { return new SynchronizedMutableIntArray(array.length(length), mutex); }
        }

        @Override public MutableIntArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedMutableIntArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override public ArrayIterator.OfInt iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfInt iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator.OfInt spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public IntStream intStream() {    // Needs to be manually synced.
            return array.intStream();
        }
        
        @Override public IntStream intParallelStream() {    // Needs to be manually synced.
            return array.intParallelStream();
        }        
        
        @Override public Stream<Integer> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Integer> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }

        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            static final long serialVersionUID = 7108555788317605319L;

            final MutableIntArray array;
            final Object mutex;

            SerializationProxy(SynchronizedMutableIntArray array) {
                this.array = array.array;
                this.mutex = array.mutex;
            }

            private Object readResolve() {
                return new SynchronizedMutableIntArray(array, mutex);
            }

            private void writeObject(ObjectOutputStream s) throws IOException {
                synchronized (mutex) {
                    s.defaultWriteObject();
                }
            }
        }
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedMutableLongArray implements MutableLongArray, UnsafeArray<Long>, Serializable {
        final MutableLongArray array;
        final Object mutex;
        
        SynchronizedMutableLongArray(MutableLongArray array) {
           this.array = array;
           this.mutex = this;
        }
        
        SynchronizedMutableLongArray(MutableLongArray array, Object mutex) {
           this.array = array;
           this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public long[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Long> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Long, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super Long> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Long get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public long getLong(int index) {
           synchronized (mutex) { return array.getLong(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsLong(long value) {
            synchronized (mutex) { return array.containsLong(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfLong(long value) {
            synchronized (mutex) { return array.indexOfLong(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfLong(long value) {
            synchronized (mutex) { return array.lastIndexOfLong(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Long> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachLong(@Nonnull LongConsumer consumer) {
            synchronized (mutex) { array.forEachLong(consumer); }
        }

        @Override public Long set(int index, Long value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public long setLong(int index, long value) {
           synchronized (mutex) { return array.setLong(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Long> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }        

        @Override public void replaceAll(@Nonnull UnaryOperator<Long> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public void replaceAllLong(@Nonnull LongUnaryOperator operator) {
            synchronized (mutex) { array.replaceAllLong(operator); }
        }

        @Override public long[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public long[] toArray(@Nonnull long[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public MutableLongArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedMutableLongArray(array.offset(offset), mutex); }
        }

        @Override public MutableLongArray length(int length) {
           synchronized (mutex) { return new SynchronizedMutableLongArray(array.length(length), mutex); }
        }

        @Override public MutableLongArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedMutableLongArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override public ArrayIterator.OfLong iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfLong iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator.OfLong spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public LongStream longStream() {    // Needs to be manually synced.
            return array.longStream();
        }
        
        @Override public LongStream longParallelStream() {    // Needs to be manually synced.
            return array.longParallelStream();
        }        
        
        @Override public Stream<Long> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Long> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
           return new SerializationProxy(this);
        }
        
        private void readObject(ObjectInputStream s) throws InvalidObjectException {
           throw new InvalidObjectException("Proxy required");
        }
        
        private static class SerializationProxy implements Serializable {
           static final long serialVersionUID = 4720852168856235946L;
        
           final MutableLongArray array;
           final Object mutex;
        
           SerializationProxy(SynchronizedMutableLongArray array) {
               this.array = array.array;
               this.mutex = array.mutex;
           }
        
           private Object readResolve() {
               return new SynchronizedMutableLongArray(array, mutex);
           }
        
           private void writeObject(ObjectOutputStream s) throws IOException {
               synchronized (mutex) {
                   s.defaultWriteObject();
               }
           }
        }
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedMutableFloatArray implements MutableFloatArray, UnsafeArray<Float>, Serializable {
        final MutableFloatArray array;
        final Object mutex;

        SynchronizedMutableFloatArray(MutableFloatArray array) {
            this.array = array;
            this.mutex = this;
        }

        SynchronizedMutableFloatArray(MutableFloatArray array, Object mutex) {
            this.array = array;
            this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public float[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Float> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Float, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super Float> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Float get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public float getFloat(int index) {
           synchronized (mutex) { return array.getFloat(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsFloat(float value) {
            synchronized (mutex) { return array.containsFloat(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfFloat(float value) {
            synchronized (mutex) { return array.indexOfFloat(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfFloat(float value) {
            synchronized (mutex) { return array.lastIndexOfFloat(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Float> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachFloat(@Nonnull FloatConsumer consumer) {
            synchronized (mutex) { array.forEachFloat(consumer); }
        }

        @Override public Float set(int index, Float value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public float setFloat(int index, float value) {
           synchronized (mutex) { return array.setFloat(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Float> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }        

        @Override public void replaceAll(@Nonnull UnaryOperator<Float> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public void replaceAllFloat(@Nonnull FloatUnaryOperator operator) {
            synchronized (mutex) { array.replaceAllFloat(operator); }
        }

        @Override public float[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public float[] toArray(@Nonnull float[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public MutableFloatArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedMutableFloatArray(array.offset(offset), mutex); }
        }

        @Override public MutableFloatArray length(int length) {
           synchronized (mutex) { return new SynchronizedMutableFloatArray(array.length(length), mutex); }
        }

        @Override public MutableFloatArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedMutableFloatArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override public ArrayIterator.OfFloat iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfFloat iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator<Float> spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public Stream<Float> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Float> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            static final long serialVersionUID = -4907124426777085984L;

            final MutableFloatArray array;
            final Object mutex;

            SerializationProxy(SynchronizedMutableFloatArray array) {
                this.array = array.array;
                this.mutex = array.mutex;
            }

            private Object readResolve() {
                return new SynchronizedMutableFloatArray(array, mutex);
            }

            private void writeObject(ObjectOutputStream s) throws IOException {
                synchronized (mutex) { s.defaultWriteObject(); }
            }
        }
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedMutableDoubleArray implements MutableDoubleArray, UnsafeArray<Double>, Serializable {
        final MutableDoubleArray array;
        final Object mutex;
        
        SynchronizedMutableDoubleArray(MutableDoubleArray array) {
           this.array = array;
           this.mutex = this;
        }
        
        SynchronizedMutableDoubleArray(MutableDoubleArray array, Object mutex) {
           this.array = array;
           this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public double[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Double> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Double, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super Double> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Double get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public double getDouble(int index) {
           synchronized (mutex) { return array.getDouble(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsDouble(double value) {
            synchronized (mutex) { return array.containsDouble(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfDouble(double value) {
            synchronized (mutex) { return array.indexOfDouble(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfDouble(double value) {
            synchronized (mutex) { return array.lastIndexOfDouble(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Double> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachDouble(@Nonnull DoubleConsumer consumer) {
            synchronized (mutex) { array.forEachDouble(consumer); }
        }

        @Override public Double set(int index, Double value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public double setDouble(int index, double value) {
           synchronized (mutex) { return array.setDouble(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Double> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }        

        @Override public void replaceAll(@Nonnull UnaryOperator<Double> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public void replaceAllDouble(@Nonnull DoubleUnaryOperator operator) {
            synchronized (mutex) { array.replaceAllDouble(operator); }
        }

        @Override public double[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public double[] toArray(@Nonnull double[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public MutableDoubleArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedMutableDoubleArray(array.offset(offset), mutex); }
        }

        @Override public MutableDoubleArray length(int length) {
           synchronized (mutex) { return new SynchronizedMutableDoubleArray(array.length(length), mutex); }
        }

        @Override public MutableDoubleArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedMutableDoubleArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override public ArrayIterator.OfDouble iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfDouble iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator.OfDouble spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public DoubleStream doubleStream() {    // Needs to be manually synced.
            return array.doubleStream();
        }
        
        @Override public DoubleStream doubleParallelStream() {    // Needs to be manually synced.
            return array.doubleParallelStream();
        }        
        
        @Override public Stream<Double> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Double> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
           return new SerializationProxy(this);
        }
        
        private void readObject(ObjectInputStream s) throws InvalidObjectException {
           throw new InvalidObjectException("Proxy required");
        }
        
        private static class SerializationProxy implements Serializable {
           static final double serialVersionUID = -8713467689833315314L;
        
           final MutableDoubleArray array;
           final Object mutex;
        
           SerializationProxy(SynchronizedMutableDoubleArray array) {
               this.array = array.array;
               this.mutex = array.mutex;
           }
        
           private Object readResolve() {
               return new SynchronizedMutableDoubleArray(array, mutex);
           }
        
           private void writeObject(ObjectOutputStream s) throws IOException {
               synchronized (mutex) {
                   s.defaultWriteObject();
               }
           }
        }
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedMutableObjectArray<E> implements MutableObjectArray<E>, UnsafeArray<E>, Serializable {
        final MutableObjectArray<E> array;
        final Object mutex;

        SynchronizedMutableObjectArray(MutableObjectArray<E> array) {
            this.array = array;
            this.mutex = this;
        }

        SynchronizedMutableObjectArray(MutableObjectArray<E> array, Object mutex) {
            this.array = array;
            this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public Object[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super E> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<E, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super E> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public E get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public @Nonnull E getNonnull(int index) {
           synchronized (mutex) { return array.getNonnull(index); }
        }

        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public void forEach(@Nonnull Consumer<? super E> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public E set(int index, E value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends E> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }

        @Override public void replaceAll(@Nonnull UnaryOperator<E> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public Object[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public <T> T[] toArray(@Nonnull T[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public MutableObjectArray<E> offset(int offset) {
           synchronized (mutex) { return new SynchronizedMutableObjectArray<>(array.offset(offset), mutex); }
        }

        @Override public MutableObjectArray<E> length(int length) {
           synchronized (mutex) { return new SynchronizedMutableObjectArray<>(array.length(length), mutex); }
        }

        @Override public MutableObjectArray<E> subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedMutableObjectArray<>(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void sort(Comparator<? super E> comparator) {
            synchronized (mutex) { array.sort(comparator); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override public void parallelSort(Comparator<? super E> comparator) {
            synchronized (mutex) { array.parallelSort(comparator); }
        }

        @Override public ArrayIterator<E> iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator<E> iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator<E> spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public Stream<E> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<E> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }

        Object writeReplace() {
            return new SerializationProxy<>(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy<E> implements Serializable {
            static final long serialVersionUID = -5214005113516421285L;

            final MutableObjectArray<E> array;
            final Object mutex;

            SerializationProxy(SynchronizedMutableObjectArray<E> array) {
                this.array = array.array;
                this.mutex = array.mutex;
            }

            private Object readResolve() {
                return new SynchronizedMutableObjectArray<>(array, mutex);
            }

            private void writeObject(ObjectOutputStream s) throws IOException {
                synchronized (mutex) { s.defaultWriteObject(); }
            }
        }
    }
    
    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedDynamicBooleanArray implements DynamicBooleanArray, UnsafeArray<Boolean>, Serializable {
        final DynamicBooleanArray array;
        final Object mutex;

        SynchronizedDynamicBooleanArray(DynamicBooleanArray array) {
            this.array = array;
            this.mutex = this;
        }

        SynchronizedDynamicBooleanArray(DynamicBooleanArray array, Object mutex) {
            this.array = array;
            this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public boolean[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Boolean> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public void setClearReservedElements(boolean flag) {
           synchronized (mutex) { array.setClearReservedElements(flag); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Boolean, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super Boolean> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Boolean get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public boolean getBoolean(int index) {
           synchronized (mutex) { return array.getBoolean(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsBoolean(boolean value) {
            synchronized (mutex) { return array.containsBoolean(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfBoolean(boolean value) {
            synchronized (mutex) { return array.indexOfBoolean(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfBoolean(boolean value) {
            synchronized (mutex) { return array.lastIndexOfBoolean(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Boolean> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachBoolean(@Nonnull BooleanConsumer consumer) {
            synchronized (mutex) { array.forEachBoolean(consumer); }
        }

        @Override public void add(Boolean value) {
           synchronized (mutex) { array.add(value); }
        }

        @Override public void addBoolean(boolean value) {
           synchronized (mutex) { array.addBoolean(value); }
        }

        @Override public void add(int index, Boolean value) {
           synchronized (mutex) { array.add(index, value); }
        }

        @Override public void addBoolean(int index, boolean value) {
           synchronized (mutex) { array.addBoolean(index, value); }
        }

        @Override public void addAll(@Nonnull Array<? extends Boolean> array) {
           synchronized (mutex) { this.array.addAll(array); }
        }

        @Override public void addAll(int index, @Nonnull Array<? extends Boolean> array) {
           synchronized (mutex) { this.array.addAll(index, array); }
        }

        @Override public Boolean set(int index, Boolean value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public boolean setBoolean(int index, boolean value) {
           synchronized (mutex) { return array.setBoolean(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Boolean> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }

        @Override public void replaceAll(@Nonnull UnaryOperator<Boolean> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public void replaceAllBoolean(@Nonnull BooleanUnaryOperator operator) {
            synchronized (mutex) { array.replaceAllBoolean(operator); }
        }

        @Override public Boolean remove(int index) {
            synchronized (mutex) { return array.remove(index); }
        }

        @Override public boolean remove(Object object) {
            synchronized (mutex) { return array.remove(object); }
        }

        @Override public boolean removeBoolean(boolean value) {
            synchronized (mutex) { return array.removeBoolean(value); }
        }

        @Override public boolean booleanRemove(int index) {
            synchronized (mutex) { return this.array.booleanRemove(index); }
        }

        @Override public boolean removeAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.removeAll(array); }
        }

        @Override public boolean retainAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.retainAll(array); }
        }

        @Override public boolean removeIf(@Nonnull Predicate<? super Boolean> filter) {
            synchronized (mutex) { return array.removeIf(filter); }
        }

        @Override public boolean removeIfBoolean(@Nonnull BooleanPredicate filter) {
            synchronized (mutex) { return array.removeIfBoolean(filter); }
        }

        @Override public void clear() {
           synchronized (mutex) { array.clear(); }
        }

        @Override public void compact() {
           synchronized (mutex) { array.compact(); }
        }

        @Override public boolean[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public boolean[] toArray(@Nonnull boolean[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public DynamicBooleanArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedDynamicBooleanArray(array.offset(offset), mutex); }
        }

        @Override public DynamicBooleanArray length(int length) {
           synchronized (mutex) { return new SynchronizedDynamicBooleanArray(array.length(length), mutex); }
        }

        @Override public DynamicBooleanArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedDynamicBooleanArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override  public void distinct() {
            synchronized (mutex) { array.distinct(); }
        }

        @Override public ArrayIterator.OfBoolean iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfBoolean iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator<Boolean> spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public Stream<Boolean> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Boolean> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            static final long serialVersionUID = 3345900774274360567L;

            final DynamicBooleanArray array;
            final Object mutex;

            SerializationProxy(SynchronizedDynamicBooleanArray array) {
                this.array = array.array;
                this.mutex = array.mutex;
            }

            private Object readResolve() {
                return new SynchronizedDynamicBooleanArray(array, mutex);
            }

            private void writeObject(ObjectOutputStream s) throws IOException {
                synchronized (mutex) { s.defaultWriteObject(); }
            }
        }
    }    

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedDynamicByteArray implements DynamicByteArray, UnsafeArray<Byte>, Serializable {
        final DynamicByteArray array;
        final Object mutex;

        SynchronizedDynamicByteArray(DynamicByteArray array) {
            this.array = array;
            this.mutex = this;
        }

        SynchronizedDynamicByteArray(DynamicByteArray array, Object mutex) {
            this.array = array;
            this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public byte[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Byte> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public void setClearReservedElements(boolean flag) {
           synchronized (mutex) { array.setClearReservedElements(flag); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Byte, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super Byte> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Byte get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public byte getByte(int index) {
           synchronized (mutex) { return array.getByte(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsByte(byte value) {
            synchronized (mutex) { return array.containsByte(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfByte(byte value) {
            synchronized (mutex) { return array.indexOfByte(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfByte(byte value) {
            synchronized (mutex) { return array.lastIndexOfByte(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Byte> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachByte(@Nonnull ByteConsumer consumer) {
            synchronized (mutex) { array.forEachByte(consumer); }
        }

        @Override public void add(Byte value) {
           synchronized (mutex) { array.add(value); }
        }

        @Override public void addByte(byte value) {
           synchronized (mutex) { array.addByte(value); }
        }

        @Override public void add(int index, Byte value) {
           synchronized (mutex) { array.add(index, value); }
        }

        @Override public void addByte(int index, byte value) {
           synchronized (mutex) { array.addByte(index, value); }
        }

        @Override public void addAll(@Nonnull Array<? extends Byte> array) {
           synchronized (mutex) { this.array.addAll(array); }
        }

        @Override public void addAll(int index, @Nonnull Array<? extends Byte> array) {
           synchronized (mutex) { this.array.addAll(index, array); }
        }

        @Override public Byte set(int index, Byte value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public byte setByte(int index, byte value) {
           synchronized (mutex) { return array.setByte(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Byte> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }

        @Override public void replaceAll(@Nonnull UnaryOperator<Byte> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public Byte remove(int index) {
            synchronized (mutex) { return array.remove(index); }
        }

        @Override public boolean remove(Object object) {
            synchronized (mutex) { return array.remove(object); }
        }

        @Override public boolean removeByte(byte value) {
            synchronized (mutex) { return array.removeByte(value); }
        }

        @Override public byte byteRemove(int index) {
            synchronized (mutex) { return this.array.byteRemove(index); }
        }

        @Override public boolean removeAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.removeAll(array); }
        }

        @Override public boolean retainAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.retainAll(array); }
        }

        @Override public boolean removeIf(@Nonnull Predicate<? super Byte> filter) {
            synchronized (mutex) { return array.removeIf(filter); }
        }

        @Override public boolean removeIfByte(@Nonnull BytePredicate filter) {
            synchronized (mutex) { return array.removeIfByte(filter); }
        }

        @Override public void clear() {
           synchronized (mutex) { array.clear(); }
        }

        @Override public void compact() {
           synchronized (mutex) { array.compact(); }
        }

        @Override public byte[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public byte[] toArray(@Nonnull byte[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public DynamicByteArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedDynamicByteArray(array.offset(offset), mutex); }
        }

        @Override public DynamicByteArray length(int length) {
           synchronized (mutex) { return new SynchronizedDynamicByteArray(array.length(length), mutex); }
        }

        @Override public DynamicByteArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedDynamicByteArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override  public void distinct() {
            synchronized (mutex) { array.distinct(); }
        }

        @Override public ArrayIterator.OfByte iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfByte iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator<Byte> spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public Stream<Byte> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Byte> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            static final long serialVersionUID = -9208059184140680021L;

            final DynamicByteArray array;
            final Object mutex;

            SerializationProxy(SynchronizedDynamicByteArray array) {
                this.array = array.array;
                this.mutex = array.mutex;
            }

            private Object readResolve() {
                return new SynchronizedDynamicByteArray(array, mutex);
            }

            private void writeObject(ObjectOutputStream s) throws IOException {
                synchronized (mutex) { s.defaultWriteObject(); }
            }
        }
    }
    
    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedDynamicCharArray implements DynamicCharArray, UnsafeArray<Character>, Serializable {
        final DynamicCharArray array;
        final Object mutex;

        SynchronizedDynamicCharArray(DynamicCharArray array) {
            this.array = array;
            this.mutex = this;
        }

        SynchronizedDynamicCharArray(DynamicCharArray array, Object mutex) {
            this.array = array;
            this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public char[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Character> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public void setClearReservedElements(boolean flag) {
           synchronized (mutex) { array.setClearReservedElements(flag); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Character, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super Character> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Character get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public char getChar(int index) {
           synchronized (mutex) { return array.getChar(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsChar(char value) {
            synchronized (mutex) { return array.containsChar(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfChar(char value) {
            synchronized (mutex) { return array.indexOfChar(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfChar(char value) {
            synchronized (mutex) { return array.lastIndexOfChar(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Character> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachChar(@Nonnull CharConsumer consumer) {
            synchronized (mutex) { array.forEachChar(consumer); }
        }

        @Override public void add(Character value) {
           synchronized (mutex) { array.add(value); }
        }

        @Override public void addChar(char value) {
           synchronized (mutex) { array.addChar(value); }
        }

        @Override public void add(int index, Character value) {
           synchronized (mutex) { array.add(index, value); }
        }

        @Override public void addChar(int index, char value) {
           synchronized (mutex) { array.addChar(index, value); }
        }

        @Override public void addAll(@Nonnull Array<? extends Character> array) {
           synchronized (mutex) { this.array.addAll(array); }
        }

        @Override public void addAll(int index, @Nonnull Array<? extends Character> array) {
           synchronized (mutex) { this.array.addAll(index, array); }
        }

        @Override public Character set(int index, Character value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public char setChar(int index, char value) {
           synchronized (mutex) { return array.setChar(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Character> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }

        @Override public void replaceAll(@Nonnull UnaryOperator<Character> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public void replaceAllChar(@Nonnull CharUnaryOperator operator) {
            synchronized (mutex) { array.replaceAllChar(operator); }
        }

        @Override public Character remove(int index) {
            synchronized (mutex) { return array.remove(index); }
        }

        @Override public boolean remove(Object object) {
            synchronized (mutex) { return array.remove(object); }
        }

        @Override public boolean removeChar(char value) {
            synchronized (mutex) { return array.removeChar(value); }
        }

        @Override public char charRemove(int index) {
            synchronized (mutex) { return this.array.charRemove(index); }
        }

        @Override public boolean removeAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.removeAll(array); }
        }

        @Override public boolean retainAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.retainAll(array); }
        }

        @Override public boolean removeIf(@Nonnull Predicate<? super Character> filter) {
            synchronized (mutex) { return array.removeIf(filter); }
        }

        @Override public boolean removeIfChar(@Nonnull CharPredicate filter) {
            synchronized (mutex) { return array.removeIfChar(filter); }
        }

        @Override public void clear() {
           synchronized (mutex) { array.clear(); }
        }

        @Override public void compact() {
           synchronized (mutex) { array.compact(); }
        }

        @Override public char[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public char[] toArray(@Nonnull char[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public DynamicCharArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedDynamicCharArray(array.offset(offset), mutex); }
        }

        @Override public DynamicCharArray length(int length) {
           synchronized (mutex) { return new SynchronizedDynamicCharArray(array.length(length), mutex); }
        }

        @Override public DynamicCharArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedDynamicCharArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override  public void distinct() {
            synchronized (mutex) { array.distinct(); }
        }

        @Override public ArrayIterator.OfChar iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfChar iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator<Character> spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public Stream<Character> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Character> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            static final long serialVersionUID = -8783682550138680070L;

            final DynamicCharArray array;
            final Object mutex;

            SerializationProxy(SynchronizedDynamicCharArray array) {
                this.array = array.array;
                this.mutex = array.mutex;
            }

            private Object readResolve() {
                return new SynchronizedDynamicCharArray(array, mutex);
            }

            private void writeObject(ObjectOutputStream s) throws IOException {
                synchronized (mutex) { s.defaultWriteObject(); }
            }
        }
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedDynamicShortArray implements DynamicShortArray, UnsafeArray<Short>, Serializable {
        final DynamicShortArray array;
        final Object mutex;

        SynchronizedDynamicShortArray(DynamicShortArray array) {
            this.array = array;
            this.mutex = this;
        }

        SynchronizedDynamicShortArray(DynamicShortArray array, Object mutex) {
            this.array = array;
            this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public short[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Short> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public void setClearReservedElements(boolean flag) {
           synchronized (mutex) { array.setClearReservedElements(flag); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Short, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super Short> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Short get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public short getShort(int index) {
           synchronized (mutex) { return array.getShort(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsShort(short value) {
            synchronized (mutex) { return array.containsShort(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfShort(short value) {
            synchronized (mutex) { return array.indexOfShort(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfShort(short value) {
            synchronized (mutex) { return array.lastIndexOfShort(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Short> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachShort(@Nonnull ShortConsumer consumer) {
            synchronized (mutex) { array.forEachShort(consumer); }
        }

        @Override public void add(Short value) {
           synchronized (mutex) { array.add(value); }
        }

        @Override public void addShort(short value) {
           synchronized (mutex) { array.addShort(value); }
        }

        @Override public void add(int index, Short value) {
           synchronized (mutex) { array.add(index, value); }
        }

        @Override public void addShort(int index, short value) {
           synchronized (mutex) { array.addShort(index, value); }
        }

        @Override public void addAll(@Nonnull Array<? extends Short> array) {
           synchronized (mutex) { this.array.addAll(array); }
        }

        @Override public void addAll(int index, @Nonnull Array<? extends Short> array) {
           synchronized (mutex) { this.array.addAll(index, array); }
        }

        @Override public Short set(int index, Short value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public short setShort(int index, short value) {
           synchronized (mutex) { return array.setShort(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Short> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }

        @Override public void replaceAll(@Nonnull UnaryOperator<Short> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public void replaceAllShort(@Nonnull ShortUnaryOperator operator) {
            synchronized (mutex) { array.replaceAllShort(operator); }
        }

        @Override public Short remove(int index) {
            synchronized (mutex) { return array.remove(index); }
        }

        @Override public boolean remove(Object object) {
            synchronized (mutex) { return array.remove(object); }
        }

        @Override public boolean removeShort(short value) {
            synchronized (mutex) { return array.removeShort(value); }
        }

        @Override public short shortRemove(int index) {
            synchronized (mutex) { return this.array.shortRemove(index); }
        }

        @Override public boolean removeAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.removeAll(array); }
        }

        @Override public boolean retainAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.retainAll(array); }
        }

        @Override public boolean removeIf(@Nonnull Predicate<? super Short> filter) {
            synchronized (mutex) { return array.removeIf(filter); }
        }

        @Override public boolean removeIfShort(@Nonnull ShortPredicate filter) {
            synchronized (mutex) { return array.removeIfShort(filter); }
        }

        @Override public void clear() {
           synchronized (mutex) { array.clear(); }
        }

        @Override public void compact() {
           synchronized (mutex) { array.compact(); }
        }

        @Override public short[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public short[] toArray(@Nonnull short[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public DynamicShortArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedDynamicShortArray(array.offset(offset), mutex); }
        }

        @Override public DynamicShortArray length(int length) {
           synchronized (mutex) { return new SynchronizedDynamicShortArray(array.length(length), mutex); }
        }

        @Override public DynamicShortArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedDynamicShortArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override  public void distinct() {
            synchronized (mutex) { array.distinct(); }
        }

        @Override public ArrayIterator.OfShort iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfShort iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator<Short> spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public Stream<Short> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Short> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            static final long serialVersionUID = 4904687719039674403L;

            final DynamicShortArray array;
            final Object mutex;

            SerializationProxy(SynchronizedDynamicShortArray array) {
                this.array = array.array;
                this.mutex = array.mutex;
            }

            private Object readResolve() {
                return new SynchronizedDynamicShortArray(array, mutex);
            }

            private void writeObject(ObjectOutputStream s) throws IOException {
                synchronized (mutex) { s.defaultWriteObject(); }
            }
        }
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedDynamicIntArray implements DynamicIntArray, UnsafeArray<Integer>, Serializable {
        final DynamicIntArray array;
        final Object mutex;

        SynchronizedDynamicIntArray(DynamicIntArray array) {
           this.array = array;
           this.mutex = this;
        }

        SynchronizedDynamicIntArray(DynamicIntArray array, Object mutex) {
           this.array = array;
           this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public int[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Integer> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public void setClearReservedElements(boolean flag) {
           synchronized (mutex) { array.setClearReservedElements(flag); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Integer, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super Integer> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Integer get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public int getInt(int index) {
           synchronized (mutex) { return array.getInt(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsInt(int value) {
            synchronized (mutex) { return array.containsInt(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfInt(int value) {
            synchronized (mutex) { return array.indexOfInt(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfInt(int value) {
            synchronized (mutex) { return array.lastIndexOfInt(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Integer> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachInt(@Nonnull IntConsumer consumer) {
            synchronized (mutex) { array.forEachInt(consumer); }
        }

        @Override public void add(Integer value) {
           synchronized (mutex) { array.add(value); }
        }

        @Override public void addInt(int value) {
           synchronized (mutex) { array.addInt(value); }
        }

        @Override public void add(int index, Integer value) {
           synchronized (mutex) { array.add(index, value); }
        }

        @Override public void addInt(int index, int value) {
           synchronized (mutex) { array.addInt(index, value); }
        }

        @Override public void addAll(@Nonnull Array<? extends Integer> array) {
           synchronized (mutex) { this.array.addAll(array); }
        }

        @Override public void addAll(int index, @Nonnull Array<? extends Integer> array) {
           synchronized (mutex) { this.array.addAll(index, array); }
        }

        @Override public Integer set(int index, Integer value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public int setInt(int index, int value) {
           synchronized (mutex) { return array.setInt(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Integer> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }

        @Override public void replaceAll(@Nonnull UnaryOperator<Integer> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public void replaceAllInt(@Nonnull IntUnaryOperator operator) {
            synchronized (mutex) { array.replaceAllInt(operator); }
        }

        @Override public Integer remove(int index) {
            synchronized (mutex) { return array.remove(index); }
        }

        @Override public boolean remove(Object object) {
            synchronized (mutex) { return array.remove(object); }
        }

        @Override public boolean removeInt(int value) {
            synchronized (mutex) { return array.removeInt(value); }
        }

        @Override public int intRemove(int index) {
            synchronized (mutex) { return this.array.intRemove(index); }
        }

        @Override public boolean removeAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.removeAll(array); }
        }

        @Override public boolean retainAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.retainAll(array); }
        }

        @Override public boolean removeIf(@Nonnull Predicate<? super Integer> filter) {
            synchronized (mutex) { return array.removeIf(filter); }
        }

        @Override public boolean removeIfInt(@Nonnull IntPredicate filter) {
            synchronized (mutex) { return array.removeIfInt(filter); }
        }

        @Override public void clear() {
           synchronized (mutex) { array.clear(); }
        }

        @Override public void compact() {
           synchronized (mutex) { array.compact(); }
        }

        @Override public int[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public int[] toArray(@Nonnull int[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public DynamicIntArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedDynamicIntArray(array.offset(offset), mutex); }
        }

        @Override public DynamicIntArray length(int length) {
           synchronized (mutex) { return new SynchronizedDynamicIntArray(array.length(length), mutex); }
        }

        @Override public DynamicIntArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedDynamicIntArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override  public void distinct() {
            synchronized (mutex) { array.distinct(); }
        }

        @Override public ArrayIterator.OfInt iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfInt iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator.OfInt spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public IntStream intStream() {    // Needs to be manually synced.
            return array.intStream();
        }
        
        @Override public IntStream intParallelStream() {    // Needs to be manually synced.
            return array.intParallelStream();
        }        
        
        @Override public Stream<Integer> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Integer> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
           return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
           throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
           static final long serialVersionUID = -3519596869473706792L;

           final DynamicIntArray array;
           final Object mutex;

           SerializationProxy(SynchronizedDynamicIntArray array) {
               this.array = array.array;
               this.mutex = array.mutex;
           }

           private Object readResolve() {
               return new SynchronizedDynamicIntArray(array, mutex);
           }

           private void writeObject(ObjectOutputStream s) throws IOException {
               synchronized (mutex) {
                   s.defaultWriteObject();
               }
           }
        }
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedDynamicLongArray implements DynamicLongArray, UnsafeArray<Long>, Serializable {
        final DynamicLongArray array;
        final Object mutex;

        SynchronizedDynamicLongArray(DynamicLongArray array) {
           this.array = array;
           this.mutex = this;
        }

        SynchronizedDynamicLongArray(DynamicLongArray array, Object mutex) {
           this.array = array;
           this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public long[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Long> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public void setClearReservedElements(boolean flag) {
           synchronized (mutex) { array.setClearReservedElements(flag); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Long, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super Long> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Long get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public long getLong(int index) {
           synchronized (mutex) { return array.getLong(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsLong(long value) {
            synchronized (mutex) { return array.containsLong(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfLong(long value) {
            synchronized (mutex) { return array.indexOfLong(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfLong(long value) {
            synchronized (mutex) { return array.lastIndexOfLong(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Long> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachLong(@Nonnull LongConsumer consumer) {
            synchronized (mutex) { array.forEachLong(consumer); }
        }

        @Override public void add(Long value) {
           synchronized (mutex) { array.add(value); }
        }

        @Override public void addLong(long value) {
           synchronized (mutex) { array.addLong(value); }
        }

        @Override public void add(int index, Long value) {
           synchronized (mutex) { array.add(index, value); }
        }

        @Override public void addLong(int index, long value) {
           synchronized (mutex) { array.addLong(index, value); }
        }

        @Override public void addAll(@Nonnull Array<? extends Long> array) {
           synchronized (mutex) { this.array.addAll(array); }
        }

        @Override public void addAll(int index, @Nonnull Array<? extends Long> array) {
           synchronized (mutex) { this.array.addAll(index, array); }
        }

        @Override public Long set(int index, Long value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public long setLong(int index, long value) {
           synchronized (mutex) { return array.setLong(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Long> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }

        @Override public void replaceAll(@Nonnull UnaryOperator<Long> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public void replaceAllLong(@Nonnull LongUnaryOperator operator) {
            synchronized (mutex) { array.replaceAllLong(operator); }
        }

        @Override public Long remove(int index) {
            synchronized (mutex) { return array.remove(index); }
        }

        @Override public boolean remove(Object object) {
            synchronized (mutex) { return array.remove(object); }
        }

        @Override public boolean removeLong(long value) {
            synchronized (mutex) { return array.removeLong(value); }
        }

        @Override public long longRemove(int index) {
            synchronized (mutex) { return this.array.longRemove(index); }
        }

        @Override public boolean removeAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.removeAll(array); }
        }

        @Override public boolean retainAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.retainAll(array); }
        }

        @Override public boolean removeIf(@Nonnull Predicate<? super Long> filter) {
            synchronized (mutex) { return array.removeIf(filter); }
        }

        @Override public boolean removeIfLong(@Nonnull LongPredicate filter) {
            synchronized (mutex) { return array.removeIfLong(filter); }
        }

        @Override public void clear() {
           synchronized (mutex) { array.clear(); }
        }

        @Override public void compact() {
           synchronized (mutex) { array.compact(); }
        }

        @Override public long[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public long[] toArray(@Nonnull long[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public DynamicLongArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedDynamicLongArray(array.offset(offset), mutex); }
        }

        @Override public DynamicLongArray length(int length) {
           synchronized (mutex) { return new SynchronizedDynamicLongArray(array.length(length), mutex); }
        }

        @Override public DynamicLongArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedDynamicLongArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override  public void distinct() {
            synchronized (mutex) { array.distinct(); }
        }

        @Override public ArrayIterator.OfLong iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfLong iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator.OfLong spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public LongStream longStream() {    // Needs to be manually synced.
            return array.longStream();
        }
        
        @Override public LongStream longParallelStream() {    // Needs to be manually synced.
            return array.longParallelStream();
        }        
        
        @Override public Stream<Long> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Long> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
           return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
           throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
           static final long serialVersionUID = -6581379766487849656L;

           final DynamicLongArray array;
           final Object mutex;

           SerializationProxy(SynchronizedDynamicLongArray array) {
               this.array = array.array;
               this.mutex = array.mutex;
           }

           private Object readResolve() {
               return new SynchronizedDynamicLongArray(array, mutex);
           }

           private void writeObject(ObjectOutputStream s) throws IOException {
               synchronized (mutex) {
                   s.defaultWriteObject();
               }
           }
        }
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedDynamicFloatArray implements DynamicFloatArray, UnsafeArray<Float>, Serializable {
        final DynamicFloatArray array;
        final Object mutex;

        SynchronizedDynamicFloatArray(DynamicFloatArray array) {
            this.array = array;
            this.mutex = this;
        }

        SynchronizedDynamicFloatArray(DynamicFloatArray array, Object mutex) {
            this.array = array;
            this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public float[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Float> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public void setClearReservedElements(boolean flag) {
           synchronized (mutex) { array.setClearReservedElements(flag); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Float, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super Float> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Float get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public float getFloat(int index) {
           synchronized (mutex) { return array.getFloat(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsFloat(float value) {
            synchronized (mutex) { return array.containsFloat(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfFloat(float value) {
            synchronized (mutex) { return array.indexOfFloat(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfFloat(float value) {
            synchronized (mutex) { return array.lastIndexOfFloat(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Float> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachFloat(@Nonnull FloatConsumer consumer) {
            synchronized (mutex) { array.forEachFloat(consumer); }
        }

        @Override public void add(Float value) {
           synchronized (mutex) { array.add(value); }
        }

        @Override public void addFloat(float value) {
           synchronized (mutex) { array.addFloat(value); }
        }

        @Override public void add(int index, Float value) {
           synchronized (mutex) { array.add(index, value); }
        }

        @Override public void addFloat(int index, float value) {
           synchronized (mutex) { array.addFloat(index, value); }
        }

        @Override public void addAll(@Nonnull Array<? extends Float> array) {
           synchronized (mutex) { this.array.addAll(array); }
        }

        @Override public void addAll(int index, @Nonnull Array<? extends Float> array) {
           synchronized (mutex) { this.array.addAll(index, array); }
        }

        @Override public Float set(int index, Float value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public float setFloat(int index, float value) {
           synchronized (mutex) { return array.setFloat(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Float> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }

        @Override public void replaceAll(@Nonnull UnaryOperator<Float> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public void replaceAllFloat(@Nonnull FloatUnaryOperator operator) {
            synchronized (mutex) { array.replaceAllFloat(operator); }
        }

        @Override public Float remove(int index) {
            synchronized (mutex) { return array.remove(index); }
        }

        @Override public boolean remove(Object object) {
            synchronized (mutex) { return array.remove(object); }
        }

        @Override public boolean removeFloat(float value) {
            synchronized (mutex) { return array.removeFloat(value); }
        }

        @Override public float floatRemove(int index) {
            synchronized (mutex) { return this.array.floatRemove(index); }
        }

        @Override public boolean removeAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.removeAll(array); }
        }

        @Override public boolean retainAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.retainAll(array); }
        }

        @Override public boolean removeIf(@Nonnull Predicate<? super Float> filter) {
            synchronized (mutex) { return array.removeIf(filter); }
        }

        @Override public boolean removeIfFloat(@Nonnull FloatPredicate filter) {
            synchronized (mutex) { return array.removeIfFloat(filter); }
        }

        @Override public void clear() {
           synchronized (mutex) { array.clear(); }
        }

        @Override public void compact() {
           synchronized (mutex) { array.compact(); }
        }

        @Override public float[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public float[] toArray(@Nonnull float[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public DynamicFloatArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedDynamicFloatArray(array.offset(offset), mutex); }
        }

        @Override public DynamicFloatArray length(int length) {
           synchronized (mutex) { return new SynchronizedDynamicFloatArray(array.length(length), mutex); }
        }

        @Override public DynamicFloatArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedDynamicFloatArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override  public void distinct() {
            synchronized (mutex) { array.distinct(); }
        }

        @Override public ArrayIterator.OfFloat iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfFloat iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator<Float> spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public Stream<Float> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Float> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            static final long serialVersionUID = 4948024420442277535L;

            final DynamicFloatArray array;
            final Object mutex;

            SerializationProxy(SynchronizedDynamicFloatArray array) {
                this.array = array.array;
                this.mutex = array.mutex;
            }

            private Object readResolve() {
                return new SynchronizedDynamicFloatArray(array, mutex);
            }

            private void writeObject(ObjectOutputStream s) throws IOException {
                synchronized (mutex) { s.defaultWriteObject(); }
            }
        }
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedDynamicDoubleArray implements DynamicDoubleArray, UnsafeArray<Double>, Serializable {
        final DynamicDoubleArray array;
        final Object mutex;

        SynchronizedDynamicDoubleArray(DynamicDoubleArray array) {
           this.array = array;
           this.mutex = this;
        }

        SynchronizedDynamicDoubleArray(DynamicDoubleArray array, Object mutex) {
           this.array = array;
           this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public double[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super Double> getComponentType() {
            synchronized (mutex) { return array.getComponentType(); }
        }

        @Override public void setClearReservedElements(boolean flag) {
           synchronized (mutex) { array.setClearReservedElements(flag); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<Double, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super Double> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public Double get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public double getDouble(int index) {
           synchronized (mutex) { return array.getDouble(index); }
        }
        
        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public boolean containsDouble(double value) {
            synchronized (mutex) { return array.containsDouble(value); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int indexOfDouble(double value) {
            synchronized (mutex) { return array.indexOfDouble(value); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public int lastIndexOfDouble(double value) {
            synchronized (mutex) { return array.lastIndexOfDouble(value); }
        }

        @Override public void forEach(@Nonnull Consumer<? super Double> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void forEachDouble(@Nonnull DoubleConsumer consumer) {
            synchronized (mutex) { array.forEachDouble(consumer); }
        }

        @Override public void add(Double value) {
           synchronized (mutex) { array.add(value); }
        }

        @Override public void addDouble(double value) {
           synchronized (mutex) { array.addDouble(value); }
        }

        @Override public void add(int index, Double value) {
           synchronized (mutex) { array.add(index, value); }
        }

        @Override public void addDouble(int index, double value) {
           synchronized (mutex) { array.addDouble(index, value); }
        }

        @Override public void addAll(@Nonnull Array<? extends Double> array) {
           synchronized (mutex) { this.array.addAll(array); }
        }

        @Override public void addAll(int index, @Nonnull Array<? extends Double> array) {
           synchronized (mutex) { this.array.addAll(index, array); }
        }

        @Override public Double set(int index, Double value) {
           synchronized (mutex) { return array.set(index, value); }
        }

        @Override public double setDouble(int index, double value) {
           synchronized (mutex) { return array.setDouble(index, value); }
        }

        @Override public void setAll(@Nonnull Array<? extends Double> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }        
        
        @Override public void replaceAll(@Nonnull UnaryOperator<Double> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public void replaceAllDouble(@Nonnull DoubleUnaryOperator operator) {
            synchronized (mutex) { array.replaceAllDouble(operator); }
        }

        @Override public Double remove(int index) {
            synchronized (mutex) { return array.remove(index); }
        }

        @Override public boolean remove(Object object) {
            synchronized (mutex) { return array.remove(object); }
        }

        @Override public boolean removeDouble(double value) {
            synchronized (mutex) { return array.removeDouble(value); }
        }

        @Override public double doubleRemove(int index) {
            synchronized (mutex) { return this.array.doubleRemove(index); }
        }

        @Override public boolean removeAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.removeAll(array); }
        }

        @Override public boolean retainAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.retainAll(array); }
        }

        @Override public boolean removeIf(@Nonnull Predicate<? super Double> filter) {
            synchronized (mutex) { return array.removeIf(filter); }
        }

        @Override public boolean removeIfDouble(@Nonnull DoublePredicate filter) {
            synchronized (mutex) { return array.removeIfDouble(filter); }
        }

        @Override public void clear() {
           synchronized (mutex) { array.clear(); }
        }

        @Override public void compact() {
           synchronized (mutex) { array.compact(); }
        }

        @Override public double[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public double[] toArray(@Nonnull double[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public DynamicDoubleArray offset(int offset) {
           synchronized (mutex) { return new SynchronizedDynamicDoubleArray(array.offset(offset), mutex); }
        }

        @Override public DynamicDoubleArray length(int length) {
           synchronized (mutex) { return new SynchronizedDynamicDoubleArray(array.length(length), mutex); }
        }

        @Override public DynamicDoubleArray subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedDynamicDoubleArray(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override  public void distinct() {
            synchronized (mutex) { array.distinct(); }
        }

        @Override public ArrayIterator.OfDouble iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator.OfDouble iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator.OfDouble spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public DoubleStream doubleStream() {    // Needs to be manually synced.
            return array.doubleStream();
        }
        
        @Override public DoubleStream doubleParallelStream() {    // Needs to be manually synced.
            return array.doubleParallelStream();
        }        
        
        @Override public Stream<Double> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<Double> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
           return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
           throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
           static final double serialVersionUID = 6396425099907082915L;

           final DynamicDoubleArray array;
           final Object mutex;

           SerializationProxy(SynchronizedDynamicDoubleArray array) {
               this.array = array.array;
               this.mutex = array.mutex;
           }

           private Object readResolve() {
               return new SynchronizedDynamicDoubleArray(array, mutex);
           }

           private void writeObject(ObjectOutputStream s) throws IOException {
               synchronized (mutex) {
                   s.defaultWriteObject();
               }
           }
        }
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class SynchronizedDynamicObjectArray<E> implements DynamicObjectArray<E>, UnsafeArray<E>, Serializable {
        final DynamicObjectArray<E> array;
        final Object mutex;

        SynchronizedDynamicObjectArray(DynamicObjectArray<E> array) {
            this.array = array;
            this.mutex = this;
        }

        SynchronizedDynamicObjectArray(DynamicObjectArray<E> array, Object mutex) {
            this.array = array;
            this.mutex = mutex;
        }

        @Override public boolean hasDirectAccess() {
            return array instanceof UnsafeArray && ((UnsafeArray) array).hasDirectAccess();
        }

        @Override public Object unsafeArray() {
            if (array instanceof UnsafeArray) {
                synchronized (mutex) { return ((UnsafeArray) array).unsafeArray(); }
            }
            throw new UnsupportedOperationException();
        }

        @Override public Object[] array() {
           synchronized (mutex) { return array.array(); }
        }

        @Override public Class<? super E> getComponentType() {
            synchronized (mutex) { return this.array.getComponentType(); }
        }

        @Override public void setClearReservedElements(boolean flag) {
           synchronized (mutex) { array.setClearReservedElements(flag); }
        }

        @Override public int characteristics() {
            synchronized (mutex) { return array.characteristics(); }
        }

        @Override public boolean hasCharacteristics(int characteristics) {
            synchronized (mutex) { return array.hasCharacteristics(characteristics); }
        }

        @Override public <T> T doAction(@Nonnull ArrayAction<E, T> action) {
            synchronized (mutex) { return array.doAction(action); }
        }

        @Override public Comparator<? super E> getComparator() {
            synchronized (mutex) { return array.getComparator(); }
        }

        @Override public int offset() {
            synchronized (mutex) { return array.offset(); }
        }

        @Override public int length() {
           synchronized (mutex) { return array.length(); }
        }

        @Override public boolean isEmpty() {
           synchronized (mutex) { return array.isEmpty(); }
        }

        @Override public E get(int index) {
           synchronized (mutex) { return array.get(index); }
        }

        @Override public @Nonnull E getNonnull(int index) {
           synchronized (mutex) { return array.getNonnull(index); }
        }

        @Override public boolean contains(Object object) {
            synchronized (mutex) { return array.contains(object); }
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.containsAll(array); }
        }

        @Override public int indexOf(Object object) {
            synchronized (mutex) { return array.indexOf(object); }
        }

        @Override public int lastIndexOf(Object object) {
            synchronized (mutex) { return array.lastIndexOf(object); }
        }

        @Override public void forEach(@Nonnull Consumer<? super E> consumer) {
           synchronized (mutex) { array.forEach(consumer); }
        }

        @Override public void add(E object) {
           synchronized (mutex) { array.add(object); }
        }

        @Override public void add(int index, E object) {
           synchronized (mutex) { array.add(index, object); }
        }

        @Override public void addAll(@Nonnull Array<? extends E> array) {
           synchronized (mutex) { this.array.addAll(array); }
        }

        @Override public void addAll(int index, @Nonnull Array<? extends E> array) {
           synchronized (mutex) { this.array.addAll(index, array); }
        }

        @Override public E set(int index, E object) {
           synchronized (mutex) { return array.set(index, object); }
        }

        @Override public void setAll(@Nonnull Array<? extends E> array) {
           synchronized (mutex) { this.array.setAll(array); }
        }

        @Override public void replaceAll(@Nonnull UnaryOperator<E> operator) {
            synchronized (mutex) { array.replaceAll(operator); }
        }

        @Override public E remove(int index) {
            synchronized (mutex) { return array.remove(index); }
        }

        @Override public boolean remove(Object object) {
            synchronized (mutex) { return array.remove(object); }
        }

        @Override public boolean removeAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.removeAll(array); }
        }

        @Override public boolean retainAll(@Nonnull Array<?> array) {
            synchronized (mutex) { return this.array.retainAll(array); }
        }

        @Override public boolean removeIf(@Nonnull Predicate<? super E> filter) {
            synchronized (mutex) { return array.removeIf(filter); }
        }

        @Override public void clear() {
           synchronized (mutex) { array.clear(); }
        }

        @Override public void compact() {
           synchronized (mutex) { array.compact(); }
        }

        @Override public Object[] toArray() {
           synchronized (mutex) { return array.toArray(); }
        }

        @Override public <T> T[] toArray(@Nonnull T[] a) {
            synchronized (mutex) { return array.toArray(a); }
        }

        @Override public DynamicObjectArray<E> offset(int offset) {
           synchronized (mutex) { return new SynchronizedDynamicObjectArray<>(array.offset(offset), mutex); }
        }

        @Override public DynamicObjectArray<E> length(int length) {
           synchronized (mutex) { return new SynchronizedDynamicObjectArray<>(array.length(length), mutex); }
        }

        @Override public DynamicObjectArray<E> subArray(int fromIndex, int toIndex) {
           synchronized (mutex) { return new SynchronizedDynamicObjectArray<>(array.subArray(fromIndex, toIndex), mutex); }
        }

        @Override public void sort() {
            synchronized (mutex) { array.sort(); }
        }

        @Override public void sort(Comparator<? super E> comparator) {
            synchronized (mutex) { array.sort(comparator); }
        }

        @Override public void parallelSort() {
            synchronized (mutex) { array.parallelSort(); }
        }

        @Override public void parallelSort(Comparator<? super E> comparator) {
            synchronized (mutex) { array.parallelSort(comparator); }
        }

        @Override  public void distinct() {
            synchronized (mutex) { array.distinct(); }
        }

        @Override  public void distinct(Comparator<? super E> comparator) {
            synchronized (mutex) { array.distinct(comparator); }
        }

        @Override public ArrayIterator<E> iterator() {   // Needs to be manually synced.
            return array.iterator();
        }

        @Override public ArrayIterator<E> iterator(int index) {  // Needs to be manually synced.
            return array.iterator(index);
        }

        @Override public Spliterator<E> spliterator() { // Needs to be manually synced.
            return array.spliterator();
        }

        @Override public Stream<E> stream() {   // Needs to be manually synced.
            return array.stream();
        }

        @Override public Stream<E> parallelStream() {   // Needs to be manually synced.
            return array.parallelStream();
        }

        @Override public int hashCode() {
            synchronized (mutex) { return array.hashCode(); }
        }

        @SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
        @Override public boolean equals(Object obj) {
            if (this == obj) return true;
            synchronized (mutex) { return array.equals(obj); }
        }

        @Override public String toString() {
            synchronized (mutex) { return array.toString(); }
        }
        
        Object writeReplace() {
            return new SerializationProxy<>(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy<E> implements Serializable {
            static final long serialVersionUID = 7064020241130583269L;

            final DynamicObjectArray<E> array;
            final Object mutex;

            SerializationProxy(SynchronizedDynamicObjectArray<E> array) {
                this.array = array.array;
                this.mutex = array.mutex;
            }

            private Object readResolve() {
                return new SynchronizedDynamicObjectArray<>(array, mutex);
            }

            private void writeObject(ObjectOutputStream s) throws IOException {
                synchronized (mutex) { s.defaultWriteObject(); }
            }
        }
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class ElementsBooleanArray implements BooleanArray, UnsafeArray<Boolean>, Serializable {
        final int offset;
        final int length;
        final boolean value;
        boolean[] array;

        ElementsBooleanArray(int length, boolean value) {
            this(0, length, value);
        }

        ElementsBooleanArray(int offset, int length, boolean value) {
            this.offset = offset;
            this.length = length;
            this.value = value;
        }

        private boolean[] createArray() {
            boolean[] a = new boolean[length];
            if (value) {
                for (int i = 0; i < length; i++) {
                    a[i] = value;
                }
            }
            return a;
        }

        @Override public boolean hasDirectAccess() {
            return true;
        }

        @Override public boolean[] unsafeArray() {
            if (array == null) {
                array = createArray();
            }
            return array;
        }

        @Override public Class<? super Boolean> getComponentType() {
            return Boolean.TYPE;
        }

        @Override public Comparator<? super Boolean> getComparator() {
            return null;
        }

        @Override
        public int characteristics() {
            return NONNULL | (length <= 1 ? SORTED | DISTINCT : 0);
        }

        @Override public int length() {
            return length;
        }

        @Override public boolean isEmpty() {
            return length == 0;
        }

        @Override public int offset() {
            return offset;
        }

        @Override public void forEach(@Nonnull Consumer<? super Boolean> consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public void forEachBoolean(@Nonnull BooleanConsumer consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public Boolean get(int index) {
            return getBoolean(index);
        }

        @Override public boolean getBoolean(int index) {
            rangeCheck(length, index);
            return value;
        }

        @Override public boolean contains(Object object) {
            return Objects.equals(value, object);
        }

        @Override public boolean containsBoolean(boolean value) {
            return value == this.value;
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            for (Object key : array) {
                if (!Objects.equals(value, key)) {
                    return false;
                }
            }
            return true;
        }

        @Override public int indexOf(Object object) {
            if (Objects.equals(value, object)) {
                return 0;
            }
            return -1;
        }

        @Override public int indexOfBoolean(boolean value) {
            return value == this.value ? 0 : -1;
        }

        @Override public int lastIndexOf(Object object) {
            if (Objects.equals(value, object)) {
                return length - 1;
            }
            return -1;
        }

        @Override public int lastIndexOfBoolean(boolean value) {
            return value == this.value ? length - 1 : -1;
        }

        @Override public boolean[] toArray() {
            return createArray();
        }

        @Override public BooleanArray offset(int offset) {
            int newOffset = this.offset + offset;
            int newLength = length - offset;
            subArrayCheck(this.offset, length, newOffset, newLength);
            return new ElementsBooleanArray(newOffset, newLength, value);
        }

        @Override public BooleanArray length(int length) {
            subArrayCheck(offset, this.length, offset, length);
            return new ElementsBooleanArray(offset, length, value);
        }

        @Override public BooleanArray subArray(int fromIndex, int toIndex) {
            int newOffset = this.offset + offset;
            subArrayCheck(this.offset, this.length, newOffset, length);
            return new ElementsBooleanArray(newOffset, length, value);
        }

        class ArrayIteratorImpl implements ArrayIterator.OfBoolean {
            int index = offset;

            ArrayIteratorImpl() {
                this(0);
            }

            ArrayIteratorImpl(int index) {
                this.index = offset + index;
            }

            @Override public boolean hasNext() {
                return index < offset + length;
            }

            @Override public boolean nextBoolean() {
                if (index == offset + length) {
                    throw new NoSuchElementException();
                }
                index++;
                return value;
            }

            @Override public void remove() {
                throw new UnsupportedOperationException();
            }

            @Override public boolean hasPrevious() {
                return index > 0;
            }

            @Override public boolean previousBoolean() {
                if (index == 0) {
                    throw new NoSuchElementException();
                }
                index--;
                return value;
            }

            @Override public int nextIndex() {
                return index;
            }

            @Override public int previousIndex() {
                return index - 1;
            }

            @Override public void setBoolean(boolean value) {
                throw new UnsupportedOperationException();
            }

            @Override public void addBoolean(boolean value) {
                throw new UnsupportedOperationException();
            }
        }

        @Override public ArrayIterator.OfBoolean iterator() {
            return new ArrayIteratorImpl();
        }

        @Override public ArrayIterator.OfBoolean iterator(int index) {
            if (index > length) {
                throw new IndexOutOfBoundsException();
            }
            return new ArrayIteratorImpl(index);
        }

        @Override public boolean[] toArray(@Nonnull boolean[] a) {
            Objects.requireNonNull(a);
            boolean[] r = a.length >= length ? a : new boolean[length];
            arraycopy(unsafeArray(), offset, r, 0, length);
            return r;
        }

        @Override public String toString() {
             int iMax = length - 1;
             if (iMax == -1) {
                 return "[]";
             }
             StringBuilder b = new StringBuilder();
             b.append('[');
             for (int i = 0; ; i++) {
                 b.append(value);
                 if (i == iMax) {
                     return b.append(']').toString();
                 }
                 b.append(", ");
             }
        }

        @Override public int hashCode() {
            int result = 1;
            int valueHash = (value ? 1231 : 1237);
            for (int i = 0; i < length; i++) {
               result = 31 * result + valueHash;
            }
            return result;
        }

        @Override public boolean equals(Object obj) {
             if (obj instanceof BooleanArray) {
                 boolean[] a = UnsafeArray.booleanReadAccess((BooleanArray) obj);
                 int l = ((BooleanArray) obj).length();
                 if (length != l) {
                     return false;
                 }
                 int o = ((BooleanArray) obj).offset();
                 for (int i = 0; i < length; i++) {
                     if (value != a[o + i]) {
                         return false;
                     }
                 }
                 return true;
             }
             return false;
        }

        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            private static final long serialVersionUID = -2734550910533009223L;

            final int length;
            final boolean value;

            SerializationProxy(ElementsBooleanArray array) {
                this.length = array.length;
                this.value = array.value;
            }

            private Object readResolve() {
                return new ElementsBooleanArray(0, length, value);
            }
        }
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class ElementsByteArray implements ByteArray, UnsafeArray<Byte>, Serializable {
        final int offset;
        final int length;
        final byte value;
        byte[] array;

        ElementsByteArray(int length, byte value) {
            this(0, length, value);
        }

        ElementsByteArray(int offset, int length, byte value) {
            this.offset = offset;
            this.length = length;
            this.value = value;
        }

        private byte[] createArray() {
            byte[] a = new byte[length];
            if (value != 0) {
                for (int i = 0; i < length; i++) {
                    a[i] = value;
                }
            }
            return a;
        }

        @Override public boolean hasDirectAccess() {
            return true;
        }

        @Override public byte[] unsafeArray() {
            if (array == null) {
                array = createArray();
            }
            return array;
        }

        @Override public Class<? super Byte> getComponentType() {
            return Byte.TYPE;
        }

        @Override public Comparator<? super Byte> getComparator() {
            return null;
        }

        @Override
        public int characteristics() {
            return NONNULL | (length <= 1 ? SORTED | DISTINCT : 0);
        }

        @Override public int length() {
            return length;
        }

        @Override public boolean isEmpty() {
            return length == 0;
        }

        @Override public int offset() {
            return offset;
        }

        @Override public void forEach(@Nonnull Consumer<? super Byte> consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public void forEachByte(@Nonnull ByteConsumer consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public Byte get(int index) {
            return getByte(index);
        }

        @Override public byte getByte(int index) {
            rangeCheck(length, index);
            return value;
        }

        @Override public boolean contains(Object object) {
            return Objects.equals(value, object);
        }

        @Override public boolean containsByte(byte value) {
            return value == this.value;
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            for (Object key : array) {
                if (!Objects.equals(value, key)) {
                    return false;
                }
            }
            return true;
        }

        @Override public int indexOf(Object object) {
            if (Objects.equals(value, object)) {
                return 0;
            }
            return -1;
        }

        @Override public int indexOfByte(byte value) {
            return value == this.value ? 0 : -1;
        }

        @Override public int lastIndexOf(Object object) {
            if (Objects.equals(value, object)) {
                return length - 1;
            }
            return -1;
        }

        @Override public int lastIndexOfByte(byte value) {
            return value == this.value ? length - 1 : -1;
        }

        @Override public byte[] toArray() {
            return createArray();
        }


        @Override public ByteArray offset(int offset) {
            int newOffset = this.offset + offset;
            int newLength = length - offset;
            subArrayCheck(this.offset, length, newOffset, newLength);
            return new ElementsByteArray(newOffset, newLength, value);
        }

        @Override public ByteArray length(int length) {
            subArrayCheck(offset, this.length, offset, length);
            return new ElementsByteArray(offset, length, value);
        }

        @Override public ByteArray subArray(int fromIndex, int toIndex) {
            int newOffset = this.offset + offset;
            subArrayCheck(this.offset, this.length, newOffset, length);
            return new ElementsByteArray(newOffset, length, value);
        }

        class ArrayIteratorImpl implements ArrayIterator.OfByte {
            int index = offset;

            ArrayIteratorImpl() {
                this(0);
            }

            ArrayIteratorImpl(int index) {
                this.index = offset + index;
            }

            @Override public boolean hasNext() {
                return index < offset + length;
            }

            @Override public byte nextByte() {
                if (index == offset + length) {
                    throw new NoSuchElementException();
                }
                index++;
                return value;
            }

            @Override public void remove() {
                throw new UnsupportedOperationException();
            }

            @Override public boolean hasPrevious() {
                return index > 0;
            }

            @Override public byte previousByte() {
                if (index == 0) {
                    throw new NoSuchElementException();
                }
                index--;
                return value;
            }

            @Override public int nextIndex() {
                return index;
            }

            @Override public int previousIndex() {
                return index - 1;
            }

            @Override public void setByte(byte value) {
                throw new UnsupportedOperationException();
            }

            @Override public void addByte(byte value) {
                throw new UnsupportedOperationException();
            }
        }

        @Override public ArrayIterator.OfByte iterator() {
            return new ArrayIteratorImpl();
        }

        @Override public ArrayIterator.OfByte iterator(int index) {
            if (index > length) {
                throw new IndexOutOfBoundsException();
            }
            return new ArrayIteratorImpl(index);
        }

        @Override public byte[] toArray(@Nonnull byte[] a) {
            Objects.requireNonNull(a);
            byte[] r = a.length >= length ? a : new byte[length];
            arraycopy(unsafeArray(), offset, r, 0, length);
            return r;
        }

        @Override public String toString() {
             int iMax = length - 1;
             if (iMax == -1) {
                 return "[]";
             }
             StringBuilder b = new StringBuilder();
             b.append('[');
             for (int i = 0; ; i++) {
                 b.append(value);
                 if (i == iMax) {
                     return b.append(']').toString();
                 }
                 b.append(", ");
             }
        }

        @Override public int hashCode() {
            int result = 1;
            for (int i = 0; i < length; i++) {
                result = 31 * result + value;
            }
            return result;
        }

        @Override public boolean equals(Object obj) {
             if (obj instanceof ByteArray) {
                 byte[] a = UnsafeArray.byteReadAccess((ByteArray) obj);
                 int l = ((ByteArray) obj).length();
                 if (length != l) {
                     return false;
                 }
                 int o = ((ByteArray) obj).offset();
                 for (int i = 0; i < length; i++) {
                     if (value != a[o + i]) {
                         return false;
                     }
                 }
                 return true;
             }
             return false;
        }

        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            private static final long serialVersionUID = 5621941582596172805L;

            final int length;
            final byte value;

            SerializationProxy(ElementsByteArray array) {
                this.length = array.length;
                this.value = array.value;
            }

            private Object readResolve() {
                return new ElementsByteArray(0, length, value);
            }
        }
    }
    
    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class ElementsCharArray implements CharArray, UnsafeArray<Character>, Serializable {
        final int offset;
        final int length;
        final char value;
        char[] array;

        ElementsCharArray(int length, char value) {
            this(0, length, value);
        }

        ElementsCharArray(int offset, int length, char value) {
            this.offset = offset;
            this.length = length;
            this.value = value;
        }

        private char[] createArray() {
            char[] a = new char[length];
            if (value != 0) {
                for (int i = 0; i < length; i++) {
                    a[i] = value;
                }
            }
            return a;
        }

        @Override public boolean hasDirectAccess() {
            return true;
        }

        @Override public char[] unsafeArray() {
            if (array == null) {
                array = createArray();
            }
            return array;
        }

        @Override public Class<? super Character> getComponentType() {
            return Character.TYPE;
        }

        @Override public Comparator<? super Character> getComparator() {
            return null;
        }

        @Override
        public int characteristics() {
            return NONNULL | (length <= 1 ? SORTED | DISTINCT : 0);
        }

        @Override public int length() {
            return length;
        }

        @Override public boolean isEmpty() {
            return length == 0;
        }

        @Override public int offset() {
            return offset;
        }

        @Override public void forEach(@Nonnull Consumer<? super Character> consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public void forEachChar(@Nonnull CharConsumer consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public Character get(int index) {
            return getChar(index);
        }

        @Override public char getChar(int index) {
            rangeCheck(length, index);
            return value;
        }

        @Override public boolean contains(Object object) {
            return Objects.equals(value, object);
        }

        @Override public boolean containsChar(char value) {
            return value == this.value;
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            for (Object key : array) {
                if (!Objects.equals(value, key)) {
                    return false;
                }
            }
            return true;
        }

        @Override public int indexOf(Object object) {
            if (Objects.equals(value, object)) {
                return 0;
            }
            return -1;
        }

        @Override public int indexOfChar(char value) {
            return value == this.value ? 0 : -1;
        }

        @Override public int lastIndexOf(Object object) {
            if (Objects.equals(value, object)) {
                return length - 1;
            }
            return -1;
        }

        @Override public int lastIndexOfChar(char value) {
            return value == this.value ? length - 1 : -1;
        }

        @Override public char[] toArray() {
            return createArray();
        }


        @Override public CharArray offset(int offset) {
            int newOffset = this.offset + offset;
            int newLength = length - offset;
            subArrayCheck(this.offset, length, newOffset, newLength);
            return new ElementsCharArray(newOffset, newLength, value);
        }

        @Override public CharArray length(int length) {
            subArrayCheck(offset, this.length, offset, length);
            return new ElementsCharArray(offset, length, value);
        }

        @Override public CharArray subArray(int fromIndex, int toIndex) {
            int newOffset = this.offset + offset;
            subArrayCheck(this.offset, this.length, newOffset, length);
            return new ElementsCharArray(newOffset, length, value);
        }

        class ArrayIteratorImpl implements ArrayIterator.OfChar {
            int index = offset;

            ArrayIteratorImpl() {
                this(0);
            }

            ArrayIteratorImpl(int index) {
                this.index = offset + index;
            }

            @Override public boolean hasNext() {
                return index < offset + length;
            }

            @Override public char nextChar() {
                if (index == offset + length) {
                    throw new NoSuchElementException();
                }
                index++;
                return value;
            }

            @Override public void remove() {
                throw new UnsupportedOperationException();
            }

            @Override public boolean hasPrevious() {
                return index > 0;
            }

            @Override public char previousChar() {
                if (index == 0) {
                    throw new NoSuchElementException();
                }
                index--;
                return value;
            }

            @Override public int nextIndex() {
                return index;
            }

            @Override public int previousIndex() {
                return index - 1;
            }

            @Override public void setChar(char value) {
                throw new UnsupportedOperationException();
            }

            @Override public void addChar(char value) {
                throw new UnsupportedOperationException();
            }
        }

        @Override public ArrayIterator.OfChar iterator() {
            return new ArrayIteratorImpl();
        }

        @Override public ArrayIterator.OfChar iterator(int index) {
            if (index > length) {
                throw new IndexOutOfBoundsException();
            }
            return new ArrayIteratorImpl(index);
        }

        @Override public char[] toArray(@Nonnull char[] a) {
            Objects.requireNonNull(a);
            char[] r = a.length >= length ? a : new char[length];
            arraycopy(unsafeArray(), offset, r, 0, length);
            return r;
        }

        @Override public String toString() {
             int iMax = length - 1;
             if (iMax == -1) {
                 return "[]";
             }
             StringBuilder b = new StringBuilder();
             b.append('[');
             for (int i = 0; ; i++) {
                 b.append(value);
                 if (i == iMax) {
                     return b.append(']').toString();
                 }
                 b.append(", ");
             }
        }

        @Override public int hashCode() {
            int result = 1;
            for (int i = 0; i < length; i++) {
                result = 31 * result + value;
            }
            return result;
        }

        @Override public boolean equals(Object obj) {
             if (obj instanceof CharArray) {
                 char[] a = UnsafeArray.charReadAccess((CharArray) obj);
                 int l = ((CharArray) obj).length();
                 if (length != l) {
                     return false;
                 }
                 int o = ((CharArray) obj).offset();
                 for (int i = 0; i < length; i++) {
                     if (value != a[o + i]) {
                         return false;
                     }
                 }
                 return true;
             }
             return false;
        }

        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            private static final long serialVersionUID = -6289900177204339053L;

            final int length;
            final char value;

            SerializationProxy(ElementsCharArray array) {
                this.length = array.length;
                this.value = array.value;
            }

            private Object readResolve() {
                return new ElementsCharArray(0, length, value);
            }
        }
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class ElementsShortArray implements ShortArray, UnsafeArray<Short>, Serializable {
        final int offset;
        final int length;
        final short value;
        short[] array;

        ElementsShortArray(int length, short value) {
            this(0, length, value);
        }

        ElementsShortArray(int offset, int length, short value) {
            this.offset = offset;
            this.length = length;
            this.value = value;
        }

        private short[] createArray() {
            short[] a = new short[length];
            if (value != 0) {
                for (int i = 0; i < length; i++) {
                    a[i] = value;
                }
            }
            return a;
        }

        @Override public boolean hasDirectAccess() {
            return true;
        }

        @Override public short[] unsafeArray() {
            if (array == null) {
                array = createArray();
            }
            return array;
        }

        @Override public Class<? super Short> getComponentType() {
            return Short.TYPE;
        }

        @Override public Comparator<? super Short> getComparator() {
            return null;
        }

        @Override
        public int characteristics() {
            return NONNULL | (length <= 1 ? SORTED | DISTINCT : 0);
        }

        @Override public int length() {
            return length;
        }

        @Override public boolean isEmpty() {
            return length == 0;
        }

        @Override public int offset() {
            return offset;
        }

        @Override public void forEach(@Nonnull Consumer<? super Short> consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public void forEachShort(@Nonnull ShortConsumer consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public Short get(int index) {
            return getShort(index);
        }

        @Override public short getShort(int index) {
            rangeCheck(length, index);
            return value;
        }

        @Override public boolean contains(Object object) {
            return Objects.equals(value, object);
        }

        @Override public boolean containsShort(short value) {
            return value == this.value;
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            for (Object key : array) {
                if (!Objects.equals(value, key)) {
                    return false;
                }
            }
            return true;
        }

        @Override public int indexOf(Object object) {
            if (Objects.equals(value, object)) {
                return 0;
            }
            return -1;
        }

        @Override public int indexOfShort(short value) {
            return value == this.value ? 0 : -1;
        }

        @Override public int lastIndexOf(Object object) {
            if (Objects.equals(value, object)) {
                return length - 1;
            }
            return -1;
        }

        @Override public int lastIndexOfShort(short value) {
            return value == this.value ? length - 1 : -1;
        }

        @Override public short[] toArray() {
            return createArray();
        }

        @Override public ShortArray offset(int offset) {
            int newOffset = this.offset + offset;
            int newLength = length - offset;
            subArrayCheck(this.offset, length, newOffset, newLength);
            return new ElementsShortArray(newOffset, newLength, value);
        }

        @Override public ShortArray length(int length) {
            subArrayCheck(offset, this.length, offset, length);
            return new ElementsShortArray(offset, length, value);
        }

        @Override public ShortArray subArray(int fromIndex, int toIndex) {
            int newOffset = this.offset + offset;
            subArrayCheck(this.offset, this.length, newOffset, length);
            return new ElementsShortArray(newOffset, length, value);
        }

        class ArrayIteratorImpl implements ArrayIterator.OfShort {
            int index = offset;

            ArrayIteratorImpl() {
                this(0);
            }

            ArrayIteratorImpl(int index) {
                this.index = offset + index;
            }

            @Override public boolean hasNext() {
                return index < offset + length;
            }

            @Override public short nextShort() {
                if (index == offset + length) {
                    throw new NoSuchElementException();
                }
                index++;
                return value;
            }

            @Override public void remove() {
                throw new UnsupportedOperationException();
            }

            @Override public boolean hasPrevious() {
                return index > 0;
            }

            @Override public short previousShort() {
                if (index == 0) {
                    throw new NoSuchElementException();
                }
                index--;
                return value;
            }

            @Override public int nextIndex() {
                return index;
            }

            @Override public int previousIndex() {
                return index - 1;
            }

            @Override public void setShort(short value) {
                throw new UnsupportedOperationException();
            }

            @Override public void addShort(short value) {
                throw new UnsupportedOperationException();
            }
        }

        @Override public ArrayIterator.OfShort iterator() {
            return new ArrayIteratorImpl();
        }

        @Override public ArrayIterator.OfShort iterator(int index) {
            if (index > length) {
                throw new IndexOutOfBoundsException();
            }
            return new ArrayIteratorImpl(index);
        }

        @Override public short[] toArray(@Nonnull short[] array) {
            Objects.requireNonNull(array);
            short[] r = array.length >= length ? array : new short[length];
            arraycopy(unsafeArray(), offset, r, 0, length);
            return r;
        }

        @Override public String toString() {
             int iMax = length - 1;
             if (iMax == -1) {
                 return "[]";
             }
             StringBuilder b = new StringBuilder();
             b.append('[');
             for (int i = 0; ; i++) {
                 b.append(value);
                 if (i == iMax) {
                     return b.append(']').toString();
                 }
                 b.append(", ");
             }
        }

        @Override public int hashCode() {
            int result = 1;
            for (int i = 0; i < length; i++) {
                result = 31 * result + value;
            }
            return result;
        }

        @Override public boolean equals(Object obj) {
             if (obj instanceof ShortArray) {
                 short[] a = UnsafeArray.shortReadAccess((ShortArray) obj);
                 int l = ((ShortArray) obj).length();
                 if (length != l) {
                     return false;
                 }
                 int o = ((ShortArray) obj).offset();
                 for (int i = 0; i < length; i++) {
                     if (value != a[o + i]) {
                         return false;
                     }
                 }
                 return true;
             }
             return false;
        }

        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            private static final long serialVersionUID = 272780045409825168L;

            final int length;
            final short value;

            SerializationProxy(ElementsShortArray array) {
                this.length = array.length;
                this.value = array.value;
            }

            private Object readResolve() {
                return new ElementsShortArray(0, length, value);
            }
        }
    }
    
    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class ElementsIntArray implements IntArray, UnsafeArray<Integer>, Serializable {
        final int offset;
        final int length;
        final int value;
        int[] array;

        ElementsIntArray(int length, int value) {
            this(0, length, value);
        }

        ElementsIntArray(int offset, int length, int value) {
            this.offset = offset;
            this.length = length;
            this.value = value;
        }

        private int[] createArray() {
            int[] a = new int[length];
            if (value != 0) {
                for (int i = 0; i < length; i++) {
                    a[i] = value;
                }
            }
            return a;
        }

        @Override public boolean hasDirectAccess() {
            return true;
        }

        @Override public int[] unsafeArray() {
            if (array == null) {
                array = createArray();
            }
            return array;
        }

        @Override public Class<? super Integer> getComponentType() {
            return Integer.TYPE;
        }

        @Override public Comparator<? super Integer> getComparator() {
            return null;
        }

        @Override
        public int characteristics() {
            return NONNULL | (length <= 1 ? SORTED | DISTINCT : 0);
        }

        @Override public int length() {
            return length;
        }

        @Override public boolean isEmpty() {
            return length == 0;
        }

        @Override public int offset() {
            return offset;
        }

        @Override public void forEach(@Nonnull Consumer<? super Integer> consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public void forEachInt(@Nonnull IntConsumer consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public Integer get(int index) {
            return getInt(index);
        }

        @Override public int getInt(int index) {
            rangeCheck(length, index);
            return value;
        }

        @Override public boolean contains(Object object) {
            return Objects.equals(value, object);
        }

        @Override public boolean containsInt(int value) {
            return value == this.value;
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            for (Object key : array) {
                if (!Objects.equals(value, key)) {
                    return false;
                }
            }
            return true;
        }

        @Override public int indexOf(Object object) {
            if (Objects.equals(value, object)) {
                return 0;
            }
            return -1;
        }

        @Override public int indexOfInt(int value) {
            return value == this.value ? 0 : -1;
        }

        @Override public int lastIndexOf(Object object) {
            if (Objects.equals(value, object)) {
                return length - 1;
            }
            return -1;
        }

        @Override public int lastIndexOfInt(int value) {
            return value == this.value ? length - 1 : -1;
        }

        @Override public int[] toArray() {
            return createArray();
        }

        @Override public IntArray offset(int offset) {
            int newOffset = this.offset + offset;
            int newLength = length - offset;
            subArrayCheck(this.offset, length, newOffset, newLength);
            return new ElementsIntArray(newOffset, newLength, value);
        }

        @Override public IntArray length(int length) {
            subArrayCheck(offset, this.length, offset, length);
            return new ElementsIntArray(offset, length, value);
        }

        @Override public IntArray subArray(int fromIndex, int toIndex) {
            int newOffset = this.offset + offset;
            subArrayCheck(this.offset, this.length, newOffset, length);
            return new ElementsIntArray(newOffset, length, value);
        }

        class ArrayIteratorImpl implements ArrayIterator.OfInt {
            int index = offset;

            ArrayIteratorImpl() {
                this(0);
            }

            ArrayIteratorImpl(int index) {
                this.index = offset + index;
            }

            @Override public boolean hasNext() {
                return index < offset + length;
            }

            @Override public int nextInt() {
                if (index == offset + length) {
                    throw new NoSuchElementException();
                }
                index++;
                return value;
            }

            @Override public void remove() {
                throw new UnsupportedOperationException();
            }

            @Override public boolean hasPrevious() {
                return index > 0;
            }

            @Override public int previousInt() {
                if (index == 0) {
                    throw new NoSuchElementException();
                }
                index--;
                return value;
            }

            @Override public int nextIndex() {
                return index;
            }

            @Override public int previousIndex() {
                return index - 1;
            }

            @Override public void setInt(int value) {
                throw new UnsupportedOperationException();
            }

            @Override public void addInt(int value) {
                throw new UnsupportedOperationException();
            }
        }

        @Override public ArrayIterator.OfInt iterator() {
            return new ArrayIteratorImpl();
        }

        @Override public ArrayIterator.OfInt iterator(int index) {
            if (index > length) {
                throw new IndexOutOfBoundsException();
            }
            return new ArrayIteratorImpl(index);
        }

        @Override
        public int[] toArray(@Nonnull int[] array) {
            Objects.requireNonNull(array);
            int[] r = array.length >= length ? array : new int[length];
            arraycopy(unsafeArray(), offset, r, 0, length);
            return r;
        }

        @Override public String toString() {
             int iMax = length - 1;
             if (iMax == -1) {
                 return "[]";
             }
             StringBuilder b = new StringBuilder();
             b.append('[');
             for (int i = 0; ; i++) {
                 b.append(value);
                 if (i == iMax) {
                     return b.append(']').toString();
                 }
                 b.append(", ");
             }
        }

        @Override public int hashCode() {
            int result = 1;
            for (int i = 0; i < length; i++) {
                result = 31 * result + value;
            }
            return result;
        }

        @Override public boolean equals(Object obj) {
             if (obj instanceof IntArray) {
                 int[] a = UnsafeArray.intReadAccess((IntArray) obj);
                 int l = ((IntArray) obj).length();
                 if (length != l) {
                     return false;
                 }
                 int o = ((IntArray) obj).offset();
                 for (int i = 0; i < length; i++) {
                     if (value != a[o + i]) {
                         return false;
                     }
                 }
                 return true;
             }
             return false;
        }

        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            private static final long serialVersionUID = 5087810323703997867L;

            final int length;
            final int value;

            SerializationProxy(ElementsIntArray array) {
                this.length = array.length;
                this.value = array.value;
            }

            private Object readResolve() {
                return new ElementsIntArray(0, length, value);
            }
        }
    }
    
    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class ElementsLongArray implements LongArray, UnsafeArray<Long>, Serializable {
        final int offset;
        final int length;
        final long value;
        long[] array;

        ElementsLongArray(int length, long value) {
            this(0, length, value);
        }

        ElementsLongArray(int offset, int length, long value) {
            this.offset = offset;
            this.length = length;
            this.value = value;
        }

        private long[] createArray() {
            long[] a = new long[length];
            if (value != 0) {
                for (int i = 0; i < length; i++) {
                    a[i] = value;
                }
            }
            return a;
        }

        @Override public boolean hasDirectAccess() {
            return true;
        }

        @Override public long[] unsafeArray() {
            if (array == null) {
                array = createArray();
            }
            return array;
        }

        @Override
        public Class<? super Long> getComponentType() {
            return Long.TYPE;
        }

        @Override public Comparator<? super Long> getComparator() {
            return null;
        }

        @Override
        public int characteristics() {
            return NONNULL | (length <= 1 ? SORTED | DISTINCT : 0);
        }

        @Override public int length() {
            return length;
        }

        @Override public boolean isEmpty() {
            return length == 0;
        }

        @Override public int offset() {
            return offset;
        }

        @Override public void forEach(@Nonnull Consumer<? super Long> consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public void forEachLong(@Nonnull LongConsumer consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public Long get(int index) {
            return getLong(index);
        }

        @Override public long getLong(int index) {
            rangeCheck(length, index);
            return value;
        }

        @Override public boolean contains(Object object) {
            return Objects.equals(value, object);
        }

        @Override public boolean containsLong(long value) {
            return value == this.value;
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            for (Object key : array) {
                if (!Objects.equals(value, key)) {
                    return false;
                }
            }
            return true;
        }

        @Override public int indexOf(Object object) {
            if (Objects.equals(value, object)) {
                return 0;
            }
            return -1;
        }

        @Override public int indexOfLong(long value) {
            return value == this.value ? 0 : -1;
        }

        @Override public int lastIndexOf(Object object) {
            if (Objects.equals(value, object)) {
                return length - 1;
            }
            return -1;
        }

        @Override public int lastIndexOfLong(long value) {
            return value == this.value ? length - 1 : -1;
        }

        @Override public long[] toArray() {
            return createArray();
        }


        @Override public LongArray offset(int offset) {
            int newOffset = this.offset + offset;
            int newLength = length - offset;
            subArrayCheck(this.offset, length, newOffset, newLength);
            return new ElementsLongArray(newOffset, newLength, value);
        }

        @Override public LongArray length(int length) {
            subArrayCheck(offset, this.length, offset, length);
            return new ElementsLongArray(offset, length, value);
        }

        @Override public LongArray subArray(int fromIndex, int toIndex) {
            int newOffset = this.offset + offset;
            subArrayCheck(this.offset, this.length, newOffset, length);
            return new ElementsLongArray(newOffset, length, value);
        }

        class ArrayIteratorImpl implements ArrayIterator.OfLong {
            int index = offset;

            ArrayIteratorImpl() {
                this(0);
            }

            ArrayIteratorImpl(int index) {
                this.index = offset + index;
            }

            @Override public boolean hasNext() {
                return index < offset + length;
            }

            @Override public long nextLong() {
                if (index == offset + length) {
                    throw new NoSuchElementException();
                }
                index++;
                return value;
            }

            @Override public void remove() {
                throw new UnsupportedOperationException();
            }

            @Override public boolean hasPrevious() {
                return index > 0;
            }

            @Override public long previousLong() {
                if (index == 0) {
                    throw new NoSuchElementException();
                }
                index--;
                return value;
            }

            @Override public int nextIndex() {
                return index;
            }

            @Override public int previousIndex() {
                return index - 1;
            }

            @Override public void setLong(long value) {
                throw new UnsupportedOperationException();
            }

            @Override public void addLong(long value) {
                throw new UnsupportedOperationException();
            }
        }

        @Override public ArrayIterator.OfLong iterator() {
            return new ArrayIteratorImpl();
        }

        @Override public ArrayIterator.OfLong iterator(int index) {
            if (index > length) {
                throw new IndexOutOfBoundsException();
            }
            return new ArrayIteratorImpl(index);
        }

        @Override
        public long[] toArray(@Nonnull long[] array) {
            Objects.requireNonNull(array);
            long[] r = array.length >= length ? array : new long[length];
            arraycopy(unsafeArray(), offset, r, 0, length);
            return r;
        }

        @Override public String toString() {
             int iMax = length - 1;
             if (iMax == -1) {
                 return "[]";
             }
             StringBuilder b = new StringBuilder();
             b.append('[');
             for (int i = 0; ; i++) {
                 b.append(value);
                 if (i == iMax) {
                     return b.append(']').toString();
                 }
                 b.append(", ");
             }
        }

        @Override public int hashCode() {
            int result = 1;
            int valueHash = (int) (value ^ (value >>> 32));
            for (int i = 0; i < length; i++) {
                result = 31 * result + valueHash;
            }
            return result;
        }

        @Override public boolean equals(Object obj) {
             if (obj instanceof LongArray) {
                 long[] a = UnsafeArray.longReadAccess((LongArray) obj);
                 int l = ((LongArray) obj).length();
                 if (length != l) {
                     return false;
                 }
                 int o = ((LongArray) obj).offset();
                 for (int i = 0; i < length; i++) {
                     if (value != a[o + i]) {
                         return false;
                     }
                 }
                 return true;
             }
             return false;
        }

        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            private static final long serialVersionUID = 2822696019071723903L;

            final int length;
            final long value;

            SerializationProxy(ElementsLongArray array) {
                this.length = array.length;
                this.value = array.value;
            }

            private Object readResolve() {
                return new ElementsLongArray(0, length, value);
            }
        }
    }
    
    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class ElementsFloatArray implements FloatArray, UnsafeArray<Float>, Serializable {
        final int offset;
        final int length;
        final float value;
        float[] array;

        ElementsFloatArray(int length, float value) {
            this(0, length, value);
        }

        ElementsFloatArray(int offset, int length, float value) {
            this.offset = offset;
            this.length = length;
            this.value = value;
        }

        private float[] createArray() {
            float[] a = new float[length];
            if (value != 0) {
                for (int i = 0; i < length; i++) {
                    a[i] = value;
                }
            }
            return a;
        }

        @Override public boolean hasDirectAccess() {
            return true;
        }

        @Override public float[] unsafeArray() {
            if (array == null) {
                array = createArray();
            }
            return array;
        }

        @Override public Class<? super Float> getComponentType() {
            return Float.TYPE;
        }

        @Override public Comparator<? super Float> getComparator() {
            return null;
        }

        @Override
        public int characteristics() {
            return NONNULL | (length <= 1 ? SORTED | DISTINCT : 0);
        }

        @Override public int length() {
            return length;
        }

        @Override public boolean isEmpty() {
            return length == 0;
        }

        @Override public int offset() {
            return offset;
        }

        @Override public void forEach(@Nonnull Consumer<? super Float> consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public void forEachFloat(@Nonnull FloatConsumer consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }
        
        @Override public Float get(int index) {
            return getFloat(index);
        }

        @Override public float getFloat(int index) {
            rangeCheck(length, index);
            return value;
        }

        @Override public boolean contains(Object object) {
            return Objects.equals(value, object);
        }

        @Override public boolean containsFloat(float value) {
            return value == this.value;
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            for (Object key : array) {
                if (!Objects.equals(value, key)) {
                    return false;
                }
            }
            return true;
        }

        @Override public int indexOf(Object object) {
            if (Objects.equals(value, object)) {
                return 0;
            }
            return -1;
        }

        @Override public int indexOfFloat(float value) {
            return value == this.value ? 0 : -1;
        }

        @Override public int lastIndexOf(Object object) {
            if (Objects.equals(value, object)) {
                return length - 1;
            }
            return -1;
        }

        @Override public int lastIndexOfFloat(float value) {
            return value == this.value ? length - 1 : -1;
        }

        @Override public float[] toArray() {
            return createArray();
        }

        @Override public FloatArray offset(int offset) {
            int newOffset = this.offset + offset;
            int newLength = length - offset;
            subArrayCheck(this.offset, length, newOffset, newLength);
            return new ElementsFloatArray(newOffset, newLength, value);
        }

        @Override public FloatArray length(int length) {
            subArrayCheck(offset, this.length, offset, length);
            return new ElementsFloatArray(offset, length, value);
        }

        @Override public FloatArray subArray(int fromIndex, int toIndex) {
            int newOffset = this.offset + offset;
            subArrayCheck(this.offset, this.length, newOffset, length);
            return new ElementsFloatArray(newOffset, length, value);
        }

        class ArrayIteratorImpl implements ArrayIterator.OfFloat {
            int index = offset;

            ArrayIteratorImpl() {
                this(0);
            }

            ArrayIteratorImpl(int index) {
                this.index = offset + index;
            }

            @Override public boolean hasNext() {
                return index < offset + length;
            }

            @Override public float nextFloat() {
                if (index == offset + length) {
                    throw new NoSuchElementException();
                }
                index++;
                return value;
            }

            @Override public void remove() {
                throw new UnsupportedOperationException();
            }

            @Override public boolean hasPrevious() {
                return index > 0;
            }

            @Override public float previousFloat() {
                if (index == 0) {
                    throw new NoSuchElementException();
                }
                index--;
                return value;
            }

            @Override public int nextIndex() {
                return index;
            }

            @Override public int previousIndex() {
                return index - 1;
            }

            @Override public void setFloat(float value) {
                throw new UnsupportedOperationException();
            }

            @Override public void addFloat(float value) {
                throw new UnsupportedOperationException();
            }
        }

        @Override public ArrayIterator.OfFloat iterator() {
            return new ArrayIteratorImpl();
        }

        @Override public ArrayIterator.OfFloat iterator(int index) {
            if (index > length) {
                throw new IndexOutOfBoundsException();
            }
            return new ArrayIteratorImpl(index);
        }

        @Override public float[] toArray(@Nonnull float[] array) {
            Objects.requireNonNull(array);
            float[] r = array.length >= length ? array : new float[length];
            arraycopy(unsafeArray(), offset, r, 0, length);
            return r;
        }

        @Override public String toString() {
             int iMax = length - 1;
             if (iMax == -1) {
                 return "[]";
             }
             StringBuilder b = new StringBuilder();
             b.append('[');
             for (int i = 0; ; i++) {
                 b.append(value);
                 if (i == iMax) {
                     return b.append(']').toString();
                 }
                 b.append(", ");
             }
        }

        @Override public int hashCode() {
            int result = 1;
            int valueHash = Float.floatToIntBits(value);
            for (int i = 0; i < length; i++) {
                result = 31 * result + valueHash;
            }
            return result;
        }

        @Override public boolean equals(Object obj) {
             if (obj instanceof FloatArray) {
                 float[] a = UnsafeArray.floatReadAccess((FloatArray) obj);
                 int l = ((FloatArray) obj).length();
                 if (length != l) {
                     return false;
                 }
                 int o = ((FloatArray) obj).offset();
                 for (int i = 0; i < length; i++) {
                     if (value != a[o + i]) {
                         return false;
                     }
                 }
                 return true;
             }
             return false;
        }

        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            private static final long serialVersionUID = 2629353948504061259L;

            final int length;
            final float value;

            SerializationProxy(ElementsFloatArray array) {
                this.length = array.length;
                this.value = array.value;
            }

            private Object readResolve() {
                return new ElementsFloatArray(0, length, value);
            }
        }
    }
    
    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class ElementsDoubleArray implements DoubleArray, UnsafeArray<Double>, Serializable {
        final int offset;
        final int length;
        final double value;
        double[] array;

        ElementsDoubleArray(int length, double value) {
            this(0, length, value);
        }

        ElementsDoubleArray(int offset, int length, double value) {
            this.offset = offset;
            this.length = length;
            this.value = value;
        }

        private double[] createArray() {
            double[] a = new double[length];
            if (value != 0) {
                for (int i = 0; i < length; i++) {
                    a[i] = value;
                }
            }
            return a;
        }

        @Override public boolean hasDirectAccess() {
            return true;
        }

        @Override public double[] unsafeArray() {
            if (array == null) {
                array = createArray();
            }
            return array;
        }

        @Override public Class<? super Double> getComponentType() {
            return Double.TYPE;
        }

        @Override public Comparator<? super Double> getComparator() {
            return null;
        }

        @Override
        public int characteristics() {
            return NONNULL | (length <= 1 ? SORTED | DISTINCT : 0);
        }

        @Override public int length() {
            return length;
        }

        @Override public boolean isEmpty() {
            return length == 0;
        }

        @Override public int offset() {
            return offset;
        }

        @Override public void forEach(@Nonnull Consumer<? super Double> consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public void forEachDouble(@Nonnull DoubleConsumer consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public Double get(int index) {
            return getDouble(index);
        }

        @Override public double getDouble(int index) {
            rangeCheck(length, index);
            return value;
        }

        @Override public boolean contains(Object object) {
            return Objects.equals(value, object);
        }

        @Override public boolean containsDouble(double value) {
            return value == this.value;
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            for (Object key : array) {
                if (!Objects.equals(value, key)) {
                    return false;
                }
            }
            return true;
        }

        @Override public int indexOf(Object object) {
            if (Objects.equals(value, object)) {
                return 0;
            }
            return -1;
        }

        @Override public int indexOfDouble(double value) {
            return value == this.value ? 0 : -1;
        }

        @Override public int lastIndexOf(Object object) {
            if (Objects.equals(value, object)) {
                return length - 1;
            }
            return -1;
        }

        @Override public int lastIndexOfDouble(double value) {
            return value == this.value ? length - 1 : -1;
        }

        @Override public double[] toArray() {
            return createArray();
        }

        @Override public DoubleArray offset(int offset) {
            int newOffset = this.offset + offset;
            int newLength = length - offset;
            subArrayCheck(this.offset, length, newOffset, newLength);
            return new ElementsDoubleArray(newOffset, newLength, value);
        }

        @Override public DoubleArray length(int length) {
            subArrayCheck(offset, this.length, offset, length);
            return new ElementsDoubleArray(offset, length, value);
        }

        @Override public DoubleArray subArray(int fromIndex, int toIndex) {
            int newOffset = this.offset + offset;
            subArrayCheck(this.offset, this.length, newOffset, length);
            return new ElementsDoubleArray(newOffset, length, value);
        }

        class ArrayIteratorImpl implements ArrayIterator.OfDouble {
            int index = offset;

            ArrayIteratorImpl() {
                this(0);
            }

            ArrayIteratorImpl(int index) {
                this.index = offset + index;
            }

            @Override public boolean hasNext() {
                return index < offset + length;
            }

            @Override public double nextDouble() {
                if (index == offset + length) {
                    throw new NoSuchElementException();
                }
                index++;
                return value;
            }

            @Override public void remove() {
                throw new UnsupportedOperationException();
            }

            @Override public boolean hasPrevious() {
                return index > 0;
            }

            @Override public double previousDouble() {
                if (index == 0) {
                    throw new NoSuchElementException();
                }
                index--;
                return value;
            }

            @Override public int nextIndex() {
                return index;
            }

            @Override public int previousIndex() {
                return index - 1;
            }

            @Override public void setDouble(double value) {
                throw new UnsupportedOperationException();
            }

            @Override public void addDouble(double value) {
                throw new UnsupportedOperationException();
            }
        }

        @Override public ArrayIterator.OfDouble iterator() {
            return new ArrayIteratorImpl();
        }

        @Override public ArrayIterator.OfDouble iterator(int index) {
            if (index > length) {
                throw new IndexOutOfBoundsException();
            }
            return new ArrayIteratorImpl(index);
        }

        @Override
        public double[] toArray(@Nonnull double[] a) {
            Objects.requireNonNull(a);
            double[] r = a.length >= length ? a : new double[length];
            arraycopy(unsafeArray(), offset, r, 0, length);
            return r;
        }

        @Override public String toString() {
             int iMax = length - 1;
             if (iMax == -1) {
                 return "[]";
             }
             StringBuilder b = new StringBuilder();
             b.append('[');
             for (int i = 0; ; i++) {
                 b.append(value);
                 if (i == iMax) {
                     return b.append(']').toString();
                 }
                 b.append(", ");
             }
        }

        @Override public int hashCode() {
            int result = 1;
            long bits = Double.doubleToLongBits(value);
            int valueHash = (int) (bits ^ (bits >>> 32));
            for (int i = 0; i < length; i++) {
                result = 31 * result + valueHash;
            }
            return result;
        }

        @Override public boolean equals(Object obj) {
             if (obj instanceof DoubleArray) {
                 double[] a = UnsafeArray.doubleReadAccess((DoubleArray) obj);
                 int l = ((DoubleArray) obj).length();
                 if (length != l) {
                     return false;
                 }
                 int o = ((DoubleArray) obj).offset();
                 for (int i = 0; i < length; i++) {
                     if (value != a[o + i]) {
                         return false;
                     }
                 }
                 return true;
             }
             return false;
        }

        Object writeReplace() {
            return new SerializationProxy(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy implements Serializable {
            private static final double serialVersionUID = 7107437812059713896L;

            final int length;
            final double value;

            SerializationProxy(ElementsDoubleArray array) {
                this.length = array.length;
                this.value = array.value;
            }

            private Object readResolve() {
                return new ElementsDoubleArray(0, length, value);
            }
        }
    }

    @SuppressWarnings({"MissingMethodJavaDoc", "MissingFieldJavaDoc"})
    private static class ElementsObjectArray<E> implements ObjectArray<E>, UnsafeArray<E>, Serializable {
        final int offset;
        final int length;
        final E value;
        E[] array;

        ElementsObjectArray(int length, E value) {
            this(0, length, value);
        }

        ElementsObjectArray(int offset, int length, E value) {
            this.offset = offset;
            this.length = length;
            this.value = value;
        }

        @Override public boolean hasDirectAccess() {
            return true;
        }

        @SuppressWarnings("unchecked")
        private E[] createArray() {
            Object[] a = new Object[length];
            if (value != null) {
                for (int i = 0; i < length; i++) {
                    a[i] = value;
                }
            }
            return (E[]) a;
        }

        @SuppressWarnings("unchecked")
        @Override public E[] unsafeArray() {
            if (array == null) {
                array = createArray();
            }
            return array;
        }

        @Override public Class<? super E> getComponentType() {
            return Object.class;
        }

        @Override public Comparator<? super E> getComparator() {
            return null;
        }

        @Override
        public int characteristics() {
            return (value != null ? NONNULL : 0) | (length <= 1 ? SORTED | DISTINCT : 0);
        }

        @Override public int length() {
            return length;
        }

        @Override public boolean isEmpty() {
            return length == 0;
        }

        @Override public int offset() {
            return offset;
        }

        @Override public void forEach(@Nonnull Consumer<? super E> consumer) {
            Objects.requireNonNull(consumer);
            for (int i = 0; i < length; i++) {
                consumer.accept(value);
            }
        }

        @Override public E get(int index) {
            rangeCheck(length, index);
            return value;
        }

        @Override public @Nonnull E getNonnull(int index) {
            rangeCheck(length, index);
            Objects.requireNonNull(value, "Assertion error, nonnull reference expected.");
            return value;
        }

        @Override public boolean contains(Object object) {
            return Objects.equals(value, object);
        }

        @Override public boolean containsAll(@Nonnull Array<?> array) {
            for (Object key : array) {
                if (!Objects.equals(value, key)) {
                    return false;
                }
            }
            return true;
        }

        @Override public int indexOf(Object object) {
            if (Objects.equals(value, object)) {
                return 0;
            }
            return -1;
        }

        @Override public int lastIndexOf(Object object) {
            if (Objects.equals(value, object)) {
                return length - 1;
            }
            return -1;
        }

        @Override public Object[] toArray() {
            return createArray();
        }

        @Override public ObjectArray<E> offset(int offset) {
            int newOffset = this.offset + offset;
            int newLength = length - offset;
            subArrayCheck(this.offset, length, newOffset, newLength);
            return new ElementsObjectArray<>(newOffset, newLength, value);
        }

        @Override public ObjectArray<E> length(int length) {
            subArrayCheck(offset, this.length, offset, length);
            return new ElementsObjectArray<>(offset, length, value);
        }

        @Override public ObjectArray<E> subArray(int fromIndex, int toIndex) {
            int newOffset = this.offset + offset;
            subArrayCheck(this.offset, this.length, newOffset, length);
            return new ElementsObjectArray<>(newOffset, length, value);
        }

        class ArrayIteratorImpl implements ArrayIterator<E> {
            int index = offset;

            ArrayIteratorImpl() {
                this(0);
            }

            ArrayIteratorImpl(int index) {
                this.index = offset + index;
            }

            @Override public boolean hasNext() {
                return index < offset + length;
            }

            @Override public E next() {
                if (index == offset + length) {
                    throw new NoSuchElementException();
                }
                index++;
                return value;
            }

            @Override public void remove() {
                throw new UnsupportedOperationException();
            }

            @Override public boolean hasPrevious() {
                return index > 0;
            }

            @Override public E previous() {
                if (index == 0) {
                    throw new NoSuchElementException();
                }
                index--;
                return value;
            }

            @Override public int nextIndex() {
                return index;
            }

            @Override public int previousIndex() {
                return index - 1;
            }

            @Override public void set(E value) {
                throw new UnsupportedOperationException();
            }

            @Override public void add(E value) {
                throw new UnsupportedOperationException();
            }
        }

        @Override public ArrayIterator<E> iterator() {
            return new ArrayIteratorImpl();
        }

        @Override public ArrayIterator<E> iterator(int index) {
            if (index > length) {
                throw new IndexOutOfBoundsException();
            }
            return new ArrayIteratorImpl(index);
        }

        @SuppressWarnings("unchecked")
        @Override public <T> T[] toArray(@Nonnull T[] array) {
            Objects.requireNonNull(array);
            T[] r = array.length >= length ? array :
                    (T[]) java.lang.reflect.Array.newInstance(array.getClass().getComponentType(), length);
            arraycopy(unsafeArray(), offset, r, 0, length);
            return r;
        }

        @Override public String toString() {
             int iMax = length - 1;
             if (iMax == -1) {
                 return "[]";
             }
             StringBuilder b = new StringBuilder();
             b.append('[');
             for (int i = 0; ; i++) {
                 b.append(value);
                 if (i == iMax) {
                     return b.append(']').toString();
                 }
                 b.append(", ");
             }
        }

        @Override public int hashCode() {
            int result = 1;
            for (int i = 0; i < length; i++) {
                result = 31 * result + (value == null ? 0 : value.hashCode());
            }
            return result;
        }

        @Override public boolean equals(Object obj) {
             if (obj instanceof ObjectArray) {
                 Object[] a = UnsafeArray.readAccess((ObjectArray) obj);
                 int l = ((ObjectArray) obj).length();
                 if (length != l) {
                     return false;
                 }
                 int o = ((ObjectArray) obj).offset();
                 for (int i = 0; i < length; i++) {
                     if (!Objects.equals(value, a[o + i])) {
                         return false;
                     }
                 }
                 return true;
             }
             return false;
        }

        Object writeReplace() {
            return new SerializationProxy<>(this);
        }

        private void readObject(ObjectInputStream s) throws InvalidObjectException {
            throw new InvalidObjectException("Proxy required");
        }

        private static class SerializationProxy<E> implements Serializable {
            private static final long serialVersionUID = -900950295819592480L;

            final int length;
            final E value;

            SerializationProxy(ElementsObjectArray<E> array) {
                this.length = array.length;
                this.value = array.value;
            }

            private Object readResolve() {
                return new ElementsObjectArray<>(0, length, value);
            }
        }
    }
}
